# Copyright (c) 2012-2018, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty


class Overrides(AWSProperty):
    props = {
        'Manifest': (dict, False),
    }


class AuthenticationConfiguration(AWSProperty):
    props = {
        'ClientId': (str, True),
        'ClientSecret': (str, True),
        'RefreshToken': (str, True),
    }


class SkillPackage(AWSProperty):
    props = {
        'Overrides': (Overrides, False),
        'S3Bucket': (str, True),
        'S3BucketRole': (str, False),
        'S3Key': (str, True),
        'S3ObjectVersion': (str, False),
    }


class Skill(AWSObject):
    resource_type = "Alexa::ASK::Skill"

    props = {
        'AuthenticationConfiguration': (AuthenticationConfiguration, True),
        'SkillPackage': (SkillPackage, True),
        'VendorId': (str, True),
    }



import time


def _tail_print(e):
    print(("%s %s %s" % (e.resource_status, e.resource_type, e.event_id)))


def get_events(conn, stackname):
    """Get the events in batches and return in chronological order"""
    next = None
    event_list = []
    while 1:
        events = conn.describe_stack_events(stackname, next)
        event_list.append(events)
        if events.next_token is None:
            break
        next = events.next_token
        time.sleep(1)
    return reversed(sum(event_list, []))


def tail(conn, stack_name, log_func=_tail_print, sleep_time=5,
         include_initial=True):
    """Show and then tail the event log"""
    # First dump the full list of events in chronological order and keep
    # track of the events we've seen already
    seen = set()
    initial_events = get_events(conn, stack_name)
    for e in initial_events:
        if include_initial:
            log_func(e)
        seen.add(e.event_id)

    # Now keep looping through and dump the new events
    while 1:
        events = get_events(conn, stack_name)
        for e in events:
            if e.event_id not in seen:
                log_func(e)
                seen.add(e.event_id)
        time.sleep(sleep_time)



# Copyright (c) 2012-2018, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, If
from .validators import backup_vault_name, double, exactly_one, json_checker


class LifecycleResourceType(AWSProperty):
    props = {
        'DeleteAfterDays': (double, False),
        'MoveToColdStorageAfterDays': (double, False),
    }


class BackupRuleResourceType(AWSProperty):
    props = {
        'CompletionWindowMinutes': (double, False),
        'Lifecycle': (LifecycleResourceType, False),
        'RecoveryPointTags': (dict, False),
        'RuleName': (str, True),
        'ScheduleExpression': (str, False),
        'StartWindowMinutes': (double, False),
        'TargetBackupVault': (str, True),
    }


class BackupPlanResourceType(AWSProperty):
    props = {
        'BackupPlanName': (str, True),
        'BackupPlanRule': ([BackupRuleResourceType], True),
    }


class BackupPlan(AWSObject):
    resource_type = "AWS::Backup::BackupPlan"

    props = {
        'BackupPlan': (BackupPlanResourceType, True),
        'BackupPlanTags': (dict, False),
    }


class ConditionResourceType(AWSProperty):
    props = {
        'ConditionKey': (str, True),
        'ConditionType': (str, True),
        'ConditionValue': (str, True),
    }


class BackupSelectionResourceType(AWSProperty):
    props = {
        'IamRoleArn': (str, True),
        'ListOfTags': ([ConditionResourceType], False),
        'Resources': ([str], False),
        'SelectionName': (str, True),
    }

    def validate(self):
        conds = [
            'ListOfTags',
            'Resources',
        ]

        def check_if(names, props):
            validated = []
            for name in names:
                validated.append(name in props and isinstance(props[name], If))
            return all(validated)

        if check_if(conds, self.properties):
            return

        exactly_one(self.__class__.__name__, self.properties, conds)


class BackupSelection(AWSObject):
    resource_type = "AWS::Backup::BackupSelection"

    props = {
        'BackupPlanId': (str, True),
        'BackupSelection': (BackupSelectionResourceType, True),
    }


class NotificationObjectType(AWSProperty):
    props = {
        'BackupVaultEvents': ([str], True),
        'SNSTopicArn': (str, True)
    }


class BackupVault(AWSObject):
    resource_type = "AWS::Backup::BackupVault"

    props = {
        'AccessPolicy': (json_checker, False),
        'BackupVaultName': (backup_vault_name, True),
        'BackupVaultTags': (dict, False),
        'EncryptionKeyArn': (str, False),
        'Notifications': (NotificationObjectType, False),
    }



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, If, Tags
from .validators import (
    elb_name, exactly_one, network_port,
    tg_healthcheck_port, integer,
    one_of, boolean
)


class LoadBalancerAttributes(AWSProperty):
    props = {
        'Key': (str, False),
        'Value': (str, False)
    }


class Certificate(AWSProperty):
    props = {
        'CertificateArn': (str, False)
    }


class AuthenticateCognitoConfig(AWSProperty):
    props = {
        "AuthenticationRequestExtraParams": (dict, False),
        "OnUnauthenticatedRequest": (str, False),
        "Scope": (str, False),
        "SessionCookieName": (str, False),
        "SessionTimeout": (integer, False),
        "UserPoolArn": (str, True),
        "UserPoolClientId": (str, True),
        "UserPoolDomain": (str, True)
    }


class AuthenticateOidcConfig(AWSProperty):
    props = {
        "AuthenticationRequestExtraParams": (dict, False),
        "AuthorizationEndpoint": (str, True),
        "ClientId": (str, True),
        "ClientSecret": (str, True),
        "Issuer": (str, True),
        "OnUnauthenticatedRequest": (str, False),
        "Scope": (str, False),
        "SessionCookieName": (str, False),
        "SessionTimeout": (integer, False),
        "TokenEndpoint": (str, True),
        "UserInfoEndpoint": (str, True)
    }


class RedirectConfig(AWSProperty):
    # https://docs.aws.amazon.com/
    # AWSCloudFormation/latest/UserGuide/
    # aws-properties-elasticloadbalancingv2-listener-redirectconfig.html
    props = {
        'Host': (str, False),
        'Path': (str, False),
        'Port': (str, False),
        'Protocol': (str, False),
        'Query': (str, False),
        'StatusCode': (str, True),
    }

    def validate(self):
        one_of(self.__class__.__name__,
               self.properties,
               'StatusCode',
               ['HTTP_301', 'HTTP_302'])


class FixedResponseConfig(AWSProperty):
    props = {
        'ContentType': (str, False),
        'MessageBody': (str, False),
        'StatusCode': (str, True),
    }

    def validate(self):
        one_of(self.__class__.__name__,
               self.properties,
               'ContentType',
               [None, 'text/plain', 'text/css', 'text/html',
                'application/javascript', 'application/json'])


class Action(AWSProperty):
    props = {
        "AuthenticateCognitoConfig": (AuthenticateCognitoConfig, False),
        "AuthenticateOidcConfig": (AuthenticateOidcConfig, False),
        "FixedResponseConfig": (FixedResponseConfig, False),
        "Order": (integer, False),
        "RedirectConfig": (RedirectConfig, False),
        "TargetGroupArn": (str, False),
        "Type": (str, True)
    }

    def validate(self):
        one_of(self.__class__.__name__,
               self.properties,
               'Type',
               ['forward', 'redirect', 'fixed-response',
                'authenticate-cognito', 'authenticate-oidc'])

        def requires(action_type, prop):
            if self.properties.get('Type') == action_type and \
                    prop not in self.properties:
                raise ValueError(
                    'Type "%s" requires definition of "%s"' % (
                        action_type, prop
                    )
                )

            if prop in self.properties and \
                    self.properties.get('Type') != action_type:
                raise ValueError(
                    'Definition of "%s" allowed only with '
                    'type "%s", was: "%s"' % (
                        prop, action_type, self.properties.get('Type')
                    )
                )

        requires('forward', 'TargetGroupArn')
        requires('redirect', 'RedirectConfig')
        requires('fixed-response', 'FixedResponseConfig')


class HostHeaderConfig(AWSProperty):
    props = {
        'Values': ([str], False),
    }


class HttpHeaderConfig(AWSProperty):
    props = {
        'HttpHeaderName': (str, False),
        'Values': ([str], False),
    }


class HttpRequestMethodConfig(AWSProperty):
    props = {
        'Values': ([str], False),
    }


class PathPatternConfig(AWSProperty):
    props = {
        'Values': ([str], False),
    }


class QueryStringKeyValue(AWSProperty):
    props = {
        'Key': (str, False),
        'Value': (str, False),
    }


class QueryStringConfig(AWSProperty):
    props = {
        'Values': ([QueryStringKeyValue], False),
    }


class SourceIpConfig(AWSProperty):
    props = {
        'Values': ([str], False),
    }


class Condition(AWSProperty):
    props = {
        'Field': (str, False),
        'HostHeaderConfig': (HostHeaderConfig, False),
        'HttpHeaderConfig': (HttpHeaderConfig, False),
        'HttpRequestMethodConfig': (HttpRequestMethodConfig, False),
        'PathPatternConfig': (PathPatternConfig, False),
        'QueryStringConfig': (QueryStringConfig, False),
        'SourceIpConfig': (SourceIpConfig, False),
        'Values': ([str], False),
    }


class Matcher(AWSProperty):
    props = {
        'HttpCode': (str, True)
    }


class SubnetMapping(AWSProperty):
    props = {
        'AllocationId': (str, True),
        'SubnetId': (str, True)
    }


class TargetGroupAttribute(AWSProperty):
    props = {
        'Key': (str, False),
        'Value': (str, False)
    }


class TargetDescription(AWSProperty):
    props = {
        'AvailabilityZone': (str, False),
        'Id': (str, True),
        'Port': (network_port, False)
    }


class Listener(AWSObject):
    resource_type = "AWS::ElasticLoadBalancingV2::Listener"

    props = {
        'Certificates': ([Certificate], False),
        'DefaultActions': ([Action], True),
        'LoadBalancerArn': (str, True),
        'Port': (network_port, True),
        'Protocol': (str, True),
        'SslPolicy': (str, False)
    }


class ListenerCertificate(AWSObject):
    resource_type = "AWS::ElasticLoadBalancingV2::ListenerCertificate"

    props = {
        'Certificates': ([Certificate], True),
        'ListenerArn': (str, True),
    }


class ListenerRule(AWSObject):
    resource_type = "AWS::ElasticLoadBalancingV2::ListenerRule"

    props = {
        'Actions': ([Action], True),
        'Conditions': ([Condition], True),
        'ListenerArn': (str, True),
        'Priority': (integer, True)
    }


TARGET_TYPE_INSTANCE = 'instance'
TARGET_TYPE_IP = 'ip'
TARGET_TYPE_LAMBDA = 'lambda'


class TargetGroup(AWSObject):
    resource_type = "AWS::ElasticLoadBalancingV2::TargetGroup"

    props = {
        'HealthCheckEnabled': (boolean, False),
        'HealthCheckIntervalSeconds': (integer, False),
        'HealthCheckPath': (str, False),
        'HealthCheckPort': (tg_healthcheck_port, False),
        'HealthCheckProtocol': (str, False),
        'HealthCheckTimeoutSeconds': (integer, False),
        'HealthyThresholdCount': (integer, False),
        'Matcher': (Matcher, False),
        'Name': (str, False),
        'Port': (network_port, False),
        'Protocol': (str, False),
        'Tags': ((Tags, list), False),
        'TargetGroupAttributes': ([TargetGroupAttribute], False),
        'Targets': ([TargetDescription], False),
        'TargetType': (str, False),
        'UnhealthyThresholdCount': (integer, False),
        'VpcId': (str, False),
    }

    def validate(self):
        one_of(self.__class__.__name__,
               self.properties,
               'TargetType',
               [
                   None,
                   TARGET_TYPE_INSTANCE,
                   TARGET_TYPE_IP,
                   TARGET_TYPE_LAMBDA
               ])

        def check_properties(action_types, props_to_check, required):

            for this_type in action_types:

                self_props = self.properties
                if self_props.get('TargetType') == this_type:

                    invalid_props = []
                    for prop in props_to_check:

                        if (prop not in self_props and required is True) or \
                                (prop in self_props and required is False):
                            invalid_props.append(prop)

                    if len(invalid_props) > 0:

                        # Make error message more readable in the default case
                        type_msg = ('Omitting TargetType' if this_type is None
                                    else 'TargetType of "%s"' % (this_type))

                        raise ValueError(
                            '%s in "%s" %s definitions of %s' % (
                                type_msg,
                                self.__class__.__name__,
                                "requires" if required is True
                                else "must not contain",
                                str(invalid_props).strip('[]')
                            ))

        # None defaults to instance as per the AWS docs
        check_properties([
                            None,
                            TARGET_TYPE_INSTANCE,
                            TARGET_TYPE_IP
                         ],
                         [
                            'Port',
                            'Protocol',
                            'VpcId'
                         ],
                         True)
        check_properties([
                            TARGET_TYPE_LAMBDA
                         ],
                         [
                            'Port',
                            'Protocol',
                            'VpcId'
                         ],
                         False)


class LoadBalancer(AWSObject):
    resource_type = "AWS::ElasticLoadBalancingV2::LoadBalancer"

    props = {
        'LoadBalancerAttributes': ([LoadBalancerAttributes], False),
        'Name': (elb_name, False),
        'Scheme': (str, False),
        'IpAddressType': (str, False),
        'SecurityGroups': (list, False),
        'SubnetMappings': ([SubnetMapping], False),
        'Subnets': (list, False),
        'Tags': ((Tags, list), False),
        'Type': (str, False),
    }

    def validate(self):
        conds = [
            'SubnetMappings',
            'Subnets',
        ]

        def check_if(names, props):
            validated = []
            for name in names:
                validated.append(name in props and isinstance(props[name], If))
            return all(validated)

        if check_if(conds, self.properties):
            return

        exactly_one(self.__class__.__name__, self.properties, conds)



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject
from . import AWSProperty
from .validators import boolean
from .validators import integer


class Connector(AWSProperty):
    props = {
        'ConnectorArn': (str, True),
        'Id': (str, True),
        'Parameters': (dict, False),
    }


class ConnectorDefinitionVersion(AWSProperty):
    props = {
        'Connectors': ([Connector], True),
    }


class ConnectorDefinition(AWSObject):
    resource_type = "AWS::Greengrass::ConnectorDefinition"

    props = {
        'InitialVersion': (ConnectorDefinitionVersion, False),
        'Name': (str, True),
        'Tags': (dict, False),
    }


class ConnectorDefinitionVersion(AWSObject):
    resource_type = "AWS::Greengrass::ConnectorDefinitionVersion"

    props = {
        'ConnectorDefinitionId': (str, True),
        'Connectors': ([Connector], True),
    }


class Core(AWSProperty):
    props = {
        'CertificateArn': (str, True),
        'Id': (str, True),
        'SyncShadow': (boolean, False),
        'ThingArn': (str, True),
    }


class CoreDefinitionVersion(AWSProperty):
    props = {
        'Cores': ([Core], True),
    }


class CoreDefinition(AWSObject):
    resource_type = "AWS::Greengrass::CoreDefinition"

    props = {
        'InitialVersion': (CoreDefinitionVersion, False),
        'Name': (str, True),
        'Tags': (dict, False),
    }


class CoreDefinitionVersion(AWSObject):
    resource_type = "AWS::Greengrass::CoreDefinitionVersion"

    props = {
        'CoreDefinitionId': (str, True),
        'Cores': ([Core], True),
    }


class Device(AWSProperty):
    props = {
        'CertificateArn': (str, True),
        'Id': (str, True),
        'SyncShadow': (boolean, False),
        'ThingArn': (str, True),
    }


class DeviceDefinitionVersion(AWSProperty):
    props = {
        'Devices': ([Device], True),
    }


class DeviceDefinition(AWSObject):
    resource_type = "AWS::Greengrass::DeviceDefinition"

    props = {
        'InitialVersion': (DeviceDefinitionVersion, False),
        'Name': (str, True),
        'Tags': (dict, False),
    }


class DeviceDefinitionVersion(AWSObject):
    resource_type = "AWS::Greengrass::DeviceDefinitionVersion"

    props = {
        'DeviceDefinitionId': (str, True),
        'Devices': ([Device], True),
    }


class RunAs(AWSProperty):
    props = {
        'Gid': (integer, False),
        'Uid': (integer, False),
    }


class Execution(AWSProperty):
    props = {
        'IsolationMode': (str, False),
        'RunAs': (RunAs, False),
    }


class DefaultConfig(AWSProperty):
    props = {
        'Execution': (Execution, True),
    }


class ResourceAccessPolicy(AWSProperty):
    props = {
        'Permission': (str, False),
        'ResourceId': (str, True),
    }


class Environment(AWSProperty):
    props = {
        'AccessSysfs': (boolean, False),
        'Execution': (Execution, False),
        'ResourceAccessPolicies': ([ResourceAccessPolicy], False),
        'Variables': (dict, False),
    }


class FunctionConfiguration(AWSProperty):
    props = {
        'EncodingType': (str, False),
        'Environment': (Environment, False),
        'ExecArgs': (str, False),
        'Executable': (str, False),
        'MemorySize': (integer, False),
        'Pinned': (boolean, False),
        'Timeout': (integer, False),
    }


class Function(AWSProperty):
    props = {
        'FunctionArn': (str, True),
        'FunctionConfiguration': (FunctionConfiguration, True),
        'Id': (str, True),
    }


class FunctionDefinitionVersion(AWSProperty):
    props = {
        'DefaultConfig': (DefaultConfig, False),
        'Functions': ([Function], True),
    }


class FunctionDefinition(AWSObject):
    resource_type = "AWS::Greengrass::FunctionDefinition"

    props = {
        'InitialVersion': (FunctionDefinitionVersion, False),
        'Name': (str, True),
        'Tags': (dict, False),
    }


class FunctionDefinitionVersion(AWSObject):
    resource_type = "AWS::Greengrass::FunctionDefinitionVersion"

    props = {
        'DefaultConfig': (DefaultConfig, False),
        'FunctionDefinitionId': (str, True),
        'Functions': ([Function], True),
    }


class GroupVersion(AWSProperty):
    props = {
        'ConnectorDefinitionVersionArn': (str, False),
        'CoreDefinitionVersionArn': (str, False),
        'DeviceDefinitionVersionArn': (str, False),
        'FunctionDefinitionVersionArn': (str, False),
        'LoggerDefinitionVersionArn': (str, False),
        'ResourceDefinitionVersionArn': (str, False),
        'SubscriptionDefinitionVersionArn': (str, False),
    }


class Group(AWSObject):
    resource_type = "AWS::Greengrass::Group"

    props = {
        'InitialVersion': (GroupVersion, False),
        'Name': (str, True),
        'RoleArn': (str, False),
        'Tags': (dict, False),
    }


class GroupVersion(AWSObject):
    resource_type = "AWS::Greengrass::GroupVersion"

    props = {
        'ConnectorDefinitionVersionArn': (str, False),
        'CoreDefinitionVersionArn': (str, False),
        'DeviceDefinitionVersionArn': (str, False),
        'FunctionDefinitionVersionArn': (str, False),
        'GroupId': (str, True),
        'LoggerDefinitionVersionArn': (str, False),
        'ResourceDefinitionVersionArn': (str, False),
        'SubscriptionDefinitionVersionArn': (str, False),
    }


class Logger(AWSProperty):
    props = {
        'Component': (str, True),
        'Id': (str, True),
        'Level': (str, True),
        'Space': (integer, False),
        'Type': (str, True),
    }


class LoggerDefinitionVersion(AWSProperty):
    props = {
        'Loggers': ([Logger], True),
    }


class LoggerDefinition(AWSObject):
    resource_type = "AWS::Greengrass::LoggerDefinition"

    props = {
        'InitialVersion': (LoggerDefinitionVersion, False),
        'Name': (str, True),
        'Tags': (dict, False),
    }


class LoggerDefinitionVersion(AWSObject):
    resource_type = "AWS::Greengrass::LoggerDefinitionVersion"

    props = {
        'LoggerDefinitionId': (str, True),
        'Loggers': ([Logger], True),
    }


class GroupOwnerSetting(AWSProperty):
    props = {
        'AutoAddGroupOwner': (boolean, True),
        'GroupOwner': (str, False),
    }


class LocalDeviceResourceData(AWSProperty):
    props = {
        'GroupOwnerSetting': (GroupOwnerSetting, False),
        'SourcePath': (str, True),
    }


class LocalVolumeResourceData(AWSProperty):
    props = {
        'DestinationPath': (str, True),
        'GroupOwnerSetting': (GroupOwnerSetting, False),
        'SourcePath': (str, True),
    }


class S3MachineLearningModelResourceData(AWSProperty):
    props = {
        'DestinationPath': (str, True),
        'S3Uri': (str, True),
    }


class SageMakerMachineLearningModelResourceData(AWSProperty):
    props = {
        'DestinationPath': (str, True),
        'SageMakerJobArn': (str, True),
    }


class SecretsManagerSecretResourceData(AWSProperty):
    props = {
        'ARN': (str, True),
        'AdditionalStagingLabelsToDownload': ([str], False),
    }


class ResourceDataContainer(AWSProperty):
    props = {
        'LocalDeviceResourceData': (LocalDeviceResourceData, False),
        'LocalVolumeResourceData': (LocalVolumeResourceData, False),
        'S3MachineLearningModelResourceData':
            (S3MachineLearningModelResourceData, False),
        'SageMakerMachineLearningModelResourceData':
            (SageMakerMachineLearningModelResourceData, False),
        'SecretsManagerSecretResourceData':
            (SecretsManagerSecretResourceData, False),
    }


class ResourceInstance(AWSProperty):
    props = {
        'Id': (str, True),
        'Name': (str, True),
        'ResourceDataContainer': (ResourceDataContainer, True),
    }


class ResourceDefinitionVersion(AWSProperty):
    props = {
        'Resources': ([ResourceInstance], True),
    }


class ResourceDefinition(AWSObject):
    resource_type = "AWS::Greengrass::ResourceDefinition"

    props = {
        'InitialVersion': (ResourceDefinitionVersion, False),
        'Name': (str, True),
        'Tags': (dict, False),
    }


class ResourceDefinitionVersion(AWSObject):
    resource_type = "AWS::Greengrass::ResourceDefinitionVersion"

    props = {
        'ResourceDefinitionId': (str, True),
        'Resources': ([ResourceInstance], True),
    }


class Subscription(AWSProperty):
    props = {
        'Id': (str, True),
        'Source': (str, True),
        'Subject': (str, True),
        'Target': (str, True),
    }


class SubscriptionDefinitionVersionProperty(AWSProperty):
    props = {
        'Subscriptions': ([Subscription], True),
    }


class SubscriptionDefinition(AWSObject):
    resource_type = "AWS::Greengrass::SubscriptionDefinition"

    props = {
        'InitialVersion': (SubscriptionDefinitionVersionProperty, False),
        'Name': (str, True),
        'Tags': (dict, False),
    }


class SubscriptionDefinitionVersion(AWSObject):
    resource_type = "AWS::Greengrass::SubscriptionDefinitionVersion"

    props = {
        'SubscriptionDefinitionId': (str, True),
        'Subscriptions': ([Subscription], True),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 10.0.0


from . import AWSObject
from . import AWSProperty
from troposphere import Tags
from .validators import boolean


class Filter(AWSProperty):
    props = {
        'Contains': ([str], False),
        'Eq': ([str], False),
        'Exists': (boolean, False),
        'Neq': ([str], False),
        'Property': (str, True),
    }


class ArchiveRule(AWSProperty):
    props = {
        'Filter': ([Filter], True),
        'RuleName': (str, True),
    }


class Analyzer(AWSObject):
    resource_type = "AWS::AccessAnalyzer::Analyzer"

    props = {
        'AnalyzerName': (str, False),
        'ArchiveRules': ([ArchiveRule], False),
        'Tags': (Tags, False),
        'Type': (str, True),
    }



# Copyright (c) 2012-2017, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty
from .validators import boolean


class SSESpecification(AWSProperty):
    props = {
        'SSEEnabled': (boolean, False),
    }


class Cluster(AWSObject):
    resource_type = "AWS::DAX::Cluster"

    props = {
        'AvailabilityZones': (str, False),
        'ClusterName': (str, False),
        'Description': (str, False),
        'IAMRoleARN': (str, True),
        'NodeType': (str, True),
        'NotificationTopicARN': (str, False),
        'ParameterGroupName': (str, False),
        'PreferredMaintenanceWindow': (str, False),
        'ReplicationFactor': (str, True),
        'SSESpecification': (SSESpecification, False),
        'SecurityGroupIds': ([str], False),
        'SubnetGroupName': (str, True),
        'Tags': (dict, False),
    }


class ParameterGroup(AWSObject):
    resource_type = "AWS::DAX::ParameterGroup"

    props = {
        'Description': (str, False),
        'ParameterGroupName': (str, False),
        'ParameterNameValues': (dict, False),
    }


class SubnetGroup(AWSObject):
    resource_type = "AWS::DAX::SubnetGroup"

    props = {
        'Description': (str, False),
        'SubnetGroupName': (str, False),
        'SubnetIds': ([str], False),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject
from troposphere import Tags
from .validators import boolean


class ResourceShare(AWSObject):
    resource_type = "AWS::RAM::ResourceShare"

    props = {
        'AllowExternalPrincipals': (boolean, False),
        'Name': (str, True),
        'Principals': ([str], False),
        'ResourceArns': ([str], False),
        'Tags': (Tags, False),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 3.3.0


from . import AWSObject
from . import AWSProperty
from troposphere import Tags


class EndpointDetails(AWSProperty):
    props = {
        'VpcEndpointId': (str, True),
    }


class IdentityProviderDetails(AWSProperty):
    props = {
        'InvocationRole': (str, True),
        'Url': (str, True),
    }


class Server(AWSObject):
    resource_type = "AWS::Transfer::Server"

    props = {
        'EndpointDetails': (EndpointDetails, False),
        'EndpointType': (str, False),
        'IdentityProviderDetails': (IdentityProviderDetails, False),
        'IdentityProviderType': (str, False),
        'LoggingRole': (str, False),
        'Tags': (Tags, False),
    }


class User(AWSObject):
    resource_type = "AWS::Transfer::User"

    props = {
        'HomeDirectory': (str, False),
        'Policy': (str, False),
        'Role': (str, True),
        'ServerId': (str, True),
        'SshPublicKeys': ([str], False),
        'Tags': (Tags, False),
        'UserName': (str, True),
    }



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSHelperFn, AWSObject, AWSProperty, Tags
from .compat import policytypes
from .validators import (
    boolean, exactly_one, integer, integer_range, double,
    network_port, positive_integer, vpn_pre_shared_key, vpn_tunnel_inside_cidr,
    vpc_endpoint_type
)

VALID_ELASTICINFERENCEACCELERATOR_TYPES = ('eia1.medium', 'eia1.large',
                                           'eia1.xlarge')


def validate_elasticinferenceaccelerator_type(
        elasticinferenceaccelerator_type):
    """Validate ElasticInferenceAccelerator for Instance"""

    if elasticinferenceaccelerator_type not in VALID_ELASTICINFERENCEACCELERATOR_TYPES:  # NOQA
        raise ValueError("Elastic Inference Accelerator Type must be one of: %s" %  # NOQA
                         ", ".join(VALID_ELASTICINFERENCEACCELERATOR_TYPES))
    return elasticinferenceaccelerator_type


class Tag(AWSProperty):
    props = {
        'Key': (str, True),
        'Value': (str, True)
    }

    def __init__(self, key=None, value=None, **kwargs):
        # provided for backward compatibility
        if key is not None:
            kwargs['Key'] = key
        if value is not None:
            kwargs['Value'] = value
        super(Tag, self).__init__(**kwargs)


class CustomerGateway(AWSObject):
    resource_type = "AWS::EC2::CustomerGateway"

    props = {
        'BgpAsn': (integer, True),
        'IpAddress': (str, True),
        'Tags': ((Tags, list), False),
        'Type': (str, True),
    }


class DHCPOptions(AWSObject):
    resource_type = "AWS::EC2::DHCPOptions"

    props = {
        'DomainName': (str, False),
        'DomainNameServers': (list, False),
        'NetbiosNameServers': (list, False),
        'NetbiosNodeType': (integer, False),
        'NtpServers': (list, False),
        'Tags': ((Tags, list), False),
    }


class EgressOnlyInternetGateway(AWSObject):
    resource_type = "AWS::EC2::EgressOnlyInternetGateway"

    props = {
        'VpcId': (str, True),
    }


class EIP(AWSObject):
    resource_type = "AWS::EC2::EIP"

    props = {
        'InstanceId': (str, False),
        'Domain': (str, False),
        'PublicIpv4Pool': (str, False),
        'Tags': (Tags, False),
    }


class EIPAssociation(AWSObject):
    resource_type = "AWS::EC2::EIPAssociation"

    props = {
        'AllocationId': (str, False),
        'EIP': (str, False),
        'InstanceId': (str, False),
        'NetworkInterfaceId': (str, False),
        'PrivateIpAddress': (str, False),
    }


class FlowLog(AWSObject):
    resource_type = "AWS::EC2::FlowLog"

    props = {
        'DeliverLogsPermissionArn': (str, False),
        'LogDestination': (str, False),
        'LogDestinationType': (str, False),
        'LogGroupName': (str, False),
        'ResourceId': (str, True),
        'ResourceType': (str, True),
        'TrafficType': (str, True),
    }


class NatGateway(AWSObject):
    resource_type = "AWS::EC2::NatGateway"

    props = {
        'AllocationId': (str, True),
        'SubnetId': (str, True),
        'Tags': ((Tags, list), False),
    }


class EBSBlockDevice(AWSProperty):
    props = {
        'DeleteOnTermination': (boolean, False),
        'Encrypted': (boolean, False),
        'KmsKeyId': (str, False),
        'Iops': (integer, False),  # Conditional
        'SnapshotId': (str, False),  # Conditional
        'VolumeSize': (integer, False),  # Conditional
        'VolumeType': (str, False),
    }


NO_DEVICE = {}


class BlockDeviceMapping(AWSProperty):
    props = {
        'DeviceName': (str, True),
        'Ebs': (EBSBlockDevice, False),  # Conditional
        'NoDevice': (dict, False),
        'VirtualName': (str, False),  # Conditional
    }


class MountPoint(AWSProperty):
    props = {
        'Device': (str, True),
        'VolumeId': (str, True),
    }


class Placement(AWSProperty):
    props = {
        'Affinity': (str, False),
        'AvailabilityZone': (str, False),
        'GroupName': (str, False),
        'HostId': (str, False),
        'Tenancy': (str, False)
    }


class CpuOptions(AWSProperty):
    props = {
        'CoreCount': (integer, False),
        'ThreadsPerCore': (integer, False),
    }


class CreditSpecification(AWSProperty):
    props = {
        'CPUCredits': (str, False),
    }


class ElasticGpuSpecification(AWSProperty):
    props = {
        'Type': (str, True),
    }


class Ipv6Addresses(AWSHelperFn):
    def __init__(self, address):
        self.data = {
            'Ipv6Address': address,
        }


class LaunchTemplateSpecification(AWSProperty):
    props = {
        'LaunchTemplateId': (str, False),
        'LaunchTemplateName': (str, False),
        'Version': (str, True),
    }


class PrivateIpAddressSpecification(AWSProperty):
    props = {
        'Primary': (boolean, True),
        'PrivateIpAddress': (str, True),
    }


class NetworkInterfaceProperty(AWSProperty):
    props = {
        'AssociatePublicIpAddress': (boolean, False),
        'DeleteOnTermination': (boolean, False),
        'Description': (str, False),
        'DeviceIndex': (integer, True),
        'GroupSet': ([str], False),
        'NetworkInterfaceId': (str, False),
        'Ipv6AddressCount': (integer, False),
        'Ipv6Addresses': ([Ipv6Addresses], False),
        'PrivateIpAddress': (str, False),
        'PrivateIpAddresses': ([PrivateIpAddressSpecification], False),
        'SecondaryPrivateIpAddressCount': (integer, False),
        'SubnetId': (str, False),
    }


class AssociationParameters(AWSProperty):
    props = {
        'Key': (str, True),
        'Value': ([str], True),
    }


class SsmAssociations(AWSProperty):
    props = {
        'AssociationParameters': ([AssociationParameters], False),
        'DocumentName': (str, True),
    }


class Host(AWSObject):
    resource_type = "AWS::EC2::Host"

    props = {
        'AutoPlacement': (str, False),
        'AvailabilityZone': (str, True),
        'HostRecovery': (str, False),
        'InstanceType': (str, True),
    }


class ElasticInferenceAccelerator(AWSProperty):
    props = {
        'Type': (validate_elasticinferenceaccelerator_type, True),
    }


class LicenseSpecification(AWSProperty):
    props = {
        'LicenseConfigurationArn': (str, True),
    }


class Instance(AWSObject):
    resource_type = "AWS::EC2::Instance"

    props = {
        'Affinity': (str, False),
        'AvailabilityZone': (str, False),
        'BlockDeviceMappings': (list, False),
        'CpuOptions': (CpuOptions, False),
        'CreditSpecification': (CreditSpecification, False),
        'DisableApiTermination': (boolean, False),
        'EbsOptimized': (boolean, False),
        'ElasticGpuSpecifications': ([ElasticGpuSpecification], False),
        'ElasticInferenceAccelerators': ([ElasticInferenceAccelerator], False),
        'HostId': (str, False),
        'IamInstanceProfile': (str, False),
        'ImageId': (str, False),
        'InstanceInitiatedShutdownBehavior': (str, False),
        'InstanceType': (str, False),
        'Ipv6AddressCount': (integer, False),
        'Ipv6Addresses': ([Ipv6Addresses], False),
        'KernelId': (str, False),
        'KeyName': (str, False),
        'LaunchTemplate': (LaunchTemplateSpecification, False),
        'LicenseSpecifications': ([LicenseSpecification], False),
        'Monitoring': (boolean, False),
        'NetworkInterfaces': ([NetworkInterfaceProperty], False),
        'PlacementGroupName': (str, False),
        'PrivateIpAddress': (str, False),
        'RamdiskId': (str, False),
        'SecurityGroupIds': (list, False),
        'SecurityGroups': (list, False),
        'SsmAssociations': ([SsmAssociations], False),
        'SourceDestCheck': (boolean, False),
        'SubnetId': (str, False),
        'Tags': ((Tags, list), False),
        'Tenancy': (str, False),
        'UserData': (str, False),
        'Volumes': (list, False),
    }


class InternetGateway(AWSObject):
    resource_type = "AWS::EC2::InternetGateway"

    props = {
        'Tags': ((Tags, list), False),
    }


class NetworkAcl(AWSObject):
    resource_type = "AWS::EC2::NetworkAcl"

    props = {
        'Tags': ((Tags, list), False),
        'VpcId': (str, True),
    }


class ICMP(AWSProperty):
    props = {
        'Code': (integer, False),
        'Type': (integer, False),
    }


class PortRange(AWSProperty):
    props = {
        'From': (network_port, False),
        'To': (network_port, False),
    }


class NetworkAclEntry(AWSObject):
    resource_type = "AWS::EC2::NetworkAclEntry"

    props = {
        'CidrBlock': (str, False),
        'Egress': (boolean, False),
        'Icmp': (ICMP, False),  # Conditional
        'Ipv6CidrBlock': (str, False),
        'NetworkAclId': (str, True),
        'PortRange': (PortRange, False),  # Conditional
        'Protocol': (network_port, True),
        'RuleAction': (str, True),
        'RuleNumber': (integer_range(1, 32766), True),
    }

    def validate(self):
        conds = [
            'CidrBlock',
            'Ipv6CidrBlock',
        ]
        exactly_one(self.__class__.__name__, self.properties, conds)


class NetworkInterface(AWSObject):
    resource_type = "AWS::EC2::NetworkInterface"

    props = {
        'Description': (str, False),
        'GroupSet': (list, False),
        'Ipv6AddressCount': (integer, False),
        'Ipv6Addresses': ([Ipv6Addresses], False),
        'PrivateIpAddress': (str, False),
        'PrivateIpAddresses': ([PrivateIpAddressSpecification], False),
        'SecondaryPrivateIpAddressCount': (integer, False),
        'SourceDestCheck': (boolean, False),
        'SubnetId': (str, True),
        'Tags': ((Tags, list), False),
    }


class NetworkInterfaceAttachment(AWSObject):
    resource_type = "AWS::EC2::NetworkInterfaceAttachment"

    props = {
        'DeleteOnTermination': (boolean, False),
        'DeviceIndex': (integer, True),
        'InstanceId': (str, True),
        'NetworkInterfaceId': (str, True),
    }


PERMISSION_INSTANCE_ATTACH = 'INSTANCE-ATTACH'
PERMISSION_EIP_ASSOCIATE = 'EIP-ASSOCIATE'


class NetworkInterfacePermission(AWSObject):
    resource_type = "AWS::EC2::NetworkInterfacePermission"

    props = {
        'AwsAccountId': (str, True),
        'NetworkInterfaceId': (str, True),
        'Permission': (str, True),
    }


class Route(AWSObject):
    resource_type = "AWS::EC2::Route"

    props = {
        'DestinationCidrBlock': (str, False),
        'DestinationIpv6CidrBlock': (str, False),
        'EgressOnlyInternetGatewayId': (str, False),
        'GatewayId': (str, False),
        'InstanceId': (str, False),
        'NatGatewayId': (str, False),
        'NetworkInterfaceId': (str, False),
        'RouteTableId': (str, True),
        'TransitGatewayId': (str, False),
        'VpcPeeringConnectionId': (str, False),
    }

    def validate(self):
        cidr_conds = [
            'DestinationCidrBlock',
            'DestinationIpv6CidrBlock',
        ]
        gateway_conds = [
            'EgressOnlyInternetGatewayId',
            'GatewayId',
            'InstanceId',
            'NatGatewayId',
            'NetworkInterfaceId',
            'TransitGatewayId',
            'VpcPeeringConnectionId'
        ]
        exactly_one(self.__class__.__name__, self.properties, cidr_conds)
        exactly_one(self.__class__.__name__, self.properties, gateway_conds)


class RouteTable(AWSObject):
    resource_type = "AWS::EC2::RouteTable"

    props = {
        'Tags': ((Tags, list), False),
        'VpcId': (str, True),
    }


def check_ports(props):
    # IpProtocol is a required field but not all values allowed require
    # ToPort and FromPort. The ones that don't need these ports are:
    ports_optional = [
        "-1",  # all protocols
        "58",  # ICMPv6
    ]
    proto = props['IpProtocol']

    if proto not in ports_optional:
        if not ('ToPort' in props and 'FromPort' in props):
            raise ValueError(
                "ToPort/FromPort must be specified for proto %s" % proto)


class SecurityGroupEgress(AWSObject):
    resource_type = "AWS::EC2::SecurityGroupEgress"

    props = {
        'CidrIp': (str, False),
        'CidrIpv6': (str, False),
        'Description': (str, False),
        'DestinationPrefixListId': (str, False),
        'DestinationSecurityGroupId': (str, False),
        'FromPort': (network_port, False),
        'GroupId': (str, True),
        'IpProtocol': (str, True),
        'ToPort': (network_port, False),
        #
        # Workaround for a bug in CloudFormation and EC2 where the
        # DestinationSecurityGroupId property is ignored causing
        # egress rules targeting a security group to be ignored.
        # Using SourceSecurityGroupId instead works fine even in
        # egress rules. AWS have known about this bug for a while.
        #
        'SourceSecurityGroupId': (str, False),
    }

    def validate(self):
        conds = [
            'CidrIp',
            'CidrIpv6',
            'DestinationPrefixListId',
            'DestinationSecurityGroupId',
        ]
        exactly_one(self.__class__.__name__, self.properties, conds)
        check_ports(self.properties)


class SecurityGroupIngress(AWSObject):
    resource_type = "AWS::EC2::SecurityGroupIngress"

    props = {
        'CidrIp': (str, False),
        'CidrIpv6': (str, False),
        'Description': (str, False),
        'FromPort': (network_port, False),
        'GroupName': (str, False),
        'GroupId': (str, False),
        'IpProtocol': (str, True),
        'SourceSecurityGroupName': (str, False),
        'SourceSecurityGroupId': (str, False),
        'SourceSecurityGroupOwnerId': (str, False),
        'ToPort': (network_port, False),
    }

    def validate(self):
        conds = [
            'CidrIp',
            'CidrIpv6',
            'SourceSecurityGroupName',
            'SourceSecurityGroupId',
        ]
        exactly_one(self.__class__.__name__, self.properties, conds)
        check_ports(self.properties)


class SecurityGroupRule(AWSProperty):
    props = {
        'CidrIp': (str, False),
        'CidrIpv6': (str, False),
        'Description': (str, False),
        'DestinationPrefixListId': (str, False),
        'DestinationSecurityGroupId': (str, False),
        'FromPort': (network_port, False),
        'IpProtocol': (str, True),
        'SourceSecurityGroupId': (str, False),
        'SourceSecurityGroupName': (str, False),
        'SourceSecurityGroupOwnerId': (str, False),
        'ToPort': (network_port, False),
    }


class SecurityGroup(AWSObject):
    resource_type = "AWS::EC2::SecurityGroup"

    props = {
        'GroupName': (str, False),
        'GroupDescription': (str, True),
        'SecurityGroupEgress': (list, False),
        'SecurityGroupIngress': (list, False),
        'VpcId': (str, False),
        'Tags': ((Tags, list), False),
    }


class Subnet(AWSObject):
    resource_type = "AWS::EC2::Subnet"

    props = {
        'AssignIpv6AddressOnCreation': (boolean, False),
        'AvailabilityZone': (str, False),
        'CidrBlock': (str, True),
        'Ipv6CidrBlock': (str, False),
        'MapPublicIpOnLaunch': (boolean, False),
        'Tags': ((Tags, list), False),
        'VpcId': (str, True),
    }

    def validate(self):
        if 'Ipv6CidrBlock' in self.properties:
            if not self.properties.get('AssignIpv6AddressOnCreation'):
                raise ValueError(
                    "If Ipv6CidrBlock is present, "
                    "AssignIpv6AddressOnCreation must be set to True"
                )


class SubnetNetworkAclAssociation(AWSObject):
    resource_type = "AWS::EC2::SubnetNetworkAclAssociation"

    props = {
        'SubnetId': (str, True),
        'NetworkAclId': (str, True),
    }


class SubnetRouteTableAssociation(AWSObject):
    resource_type = "AWS::EC2::SubnetRouteTableAssociation"

    props = {
        'RouteTableId': (str, True),
        'SubnetId': (str, True),
    }


class Volume(AWSObject):
    resource_type = "AWS::EC2::Volume"

    props = {
        'AutoEnableIO': (boolean, False),
        'AvailabilityZone': (str, True),
        'Encrypted': (boolean, False),
        'Iops': (positive_integer, False),
        'KmsKeyId': (str, False),
        'Size': (positive_integer, False),
        'SnapshotId': (str, False),
        'Tags': ((Tags, list), False),
        'VolumeType': (str, False),
    }


class VolumeAttachment(AWSObject):
    resource_type = "AWS::EC2::VolumeAttachment"

    props = {
        'Device': (str, True),
        'InstanceId': (str, True),
        'VolumeId': (str, True),
    }


def instance_tenancy(value):
    valid = ['default', 'dedicated']
    if value not in valid:
        raise ValueError('InstanceTenancy needs to be one of %r' % valid)
    return value


class VPC(AWSObject):
    resource_type = "AWS::EC2::VPC"

    props = {
        'CidrBlock': (str, True),
        'EnableDnsSupport': (boolean, False),
        'EnableDnsHostnames': (boolean, False),
        'InstanceTenancy': (instance_tenancy, False),
        'Tags': ((Tags, list), False),
    }


class VPCDHCPOptionsAssociation(AWSObject):
    resource_type = "AWS::EC2::VPCDHCPOptionsAssociation"

    props = {
        'DhcpOptionsId': (str, True),
        'VpcId': (str, True),
    }


class VPCEndpoint(AWSObject):
    resource_type = "AWS::EC2::VPCEndpoint"

    props = {
        'PolicyDocument': (policytypes, False),
        'PrivateDnsEnabled': (boolean, False),
        'RouteTableIds': ([str], False),
        'SecurityGroupIds': ([str], False),
        'ServiceName': (str, True),
        'SubnetIds': ([str], False),
        'VpcEndpointType': (vpc_endpoint_type, False),
        'VpcId': (str, True),
    }


class VPCEndpointConnectionNotification(AWSObject):
    resource_type = "AWS::EC2::VPCEndpointConnectionNotification"

    props = {
        'ConnectionEvents': ([str], True),
        'ConnectionNotificationArn': (str, True),
        'ServiceId': (str, False),
        'VPCEndpointId': (str, False),
    }


class VPCEndpointService(AWSObject):
    resource_type = "AWS::EC2::VPCEndpointService"

    props = {
        'AcceptanceRequired': (boolean, False),
        'NetworkLoadBalancerArns': ([str], True),
    }


class VPCEndpointServicePermissions(AWSObject):
    resource_type = "AWS::EC2::VPCEndpointServicePermissions"

    props = {
        'AllowedPrincipals': ([str], False),
        'ServiceId': (str, True),
    }


class VPCGatewayAttachment(AWSObject):
    resource_type = "AWS::EC2::VPCGatewayAttachment"

    props = {
        'InternetGatewayId': (str, False),
        'VpcId': (str, True),
        'VpnGatewayId': (str, False),
    }


class VpnTunnelOptionsSpecification(AWSProperty):
    props = {
        'PreSharedKey': (vpn_pre_shared_key, False),
        'TunnelInsideCidr': (vpn_tunnel_inside_cidr, False),
    }


class VPNConnection(AWSObject):
    resource_type = "AWS::EC2::VPNConnection"

    props = {
        'CustomerGatewayId': (str, True),
        'StaticRoutesOnly': (boolean, False),
        'Tags': ((Tags, list), False),
        'TransitGatewayId': (str, False),
        'Type': (str, True),
        'VpnGatewayId': (str, False),
        'VpnTunnelOptionsSpecifications':
            ([VpnTunnelOptionsSpecification], False),
    }

    def validate(self):
        conds = [
            'VpnGatewayId',
            'TransitGatewayId',
        ]
        exactly_one(self.__class__.__name__, self.properties, conds)


class VPNConnectionRoute(AWSObject):
    resource_type = "AWS::EC2::VPNConnectionRoute"

    props = {
        'DestinationCidrBlock': (str, True),
        'VpnConnectionId': (str, True),
    }


class VPNGateway(AWSObject):
    resource_type = "AWS::EC2::VPNGateway"

    props = {
        'AmazonSideAsn': (positive_integer, False),
        'Type': (str, True),
        'Tags': ((Tags, list), False),
    }


class VPNGatewayRoutePropagation(AWSObject):
    resource_type = "AWS::EC2::VPNGatewayRoutePropagation"

    props = {
        'RouteTableIds': ([str], True),
        'VpnGatewayId': (str, True),
    }


class VPCPeeringConnection(AWSObject):
    resource_type = "AWS::EC2::VPCPeeringConnection"

    props = {
        'PeerVpcId': (str, True),
        'VpcId': (str, True),
        'Tags': ((Tags, list), False),
        'PeerRegion': (str, False),
        'PeerOwnerId': (str, False),
        'PeerRoleArn': (str, False),
    }


class Monitoring(AWSProperty):
    props = {
        'Enabled': (boolean, False),
    }


class NetworkInterfaces(AWSProperty):
    props = {
        'AssociatePublicIpAddress': (boolean, False),
        'DeleteOnTermination': (boolean, False),
        'Description': (str, False),
        'DeviceIndex': (integer, True),
        'Groups': ([str], False),
        'InterfaceType': (str, False),
        'Ipv6AddressCount': (integer, False),
        'Ipv6Addresses': ([Ipv6Addresses], False),
        'NetworkInterfaceId': (str, False),
        'PrivateIpAddresses': ([PrivateIpAddressSpecification], False),
        'SecondaryPrivateIpAddressCount': (integer, False),
        'SubnetId': (str, False),
    }


class SecurityGroups(AWSProperty):
    props = {
        'GroupId': (str, False),
    }


class IamInstanceProfile(AWSProperty):
    props = {
        'Arn': (str, False),
    }


class SpotFleetTagSpecification(AWSProperty):
    props = {
        'ResourceType': (str, True),
        'Tags': ((Tags, list), False),
    }


class LaunchSpecifications(AWSProperty):
    props = {
        'BlockDeviceMappings': ([BlockDeviceMapping], False),
        'EbsOptimized': (boolean, False),
        'IamInstanceProfile': (IamInstanceProfile, False),
        'ImageId': (str, True),
        'InstanceType': (str, True),
        'KernelId': (str, False),
        'KeyName': (str, False),
        'Monitoring': (Monitoring, False),
        'NetworkInterfaces': ([NetworkInterfaces], False),
        'Placement': (Placement, False),
        'RamdiskId': (str, False),
        'SecurityGroups': ([SecurityGroups], False),
        'SpotPrice': (str, False),
        'SubnetId': (str, False),
        'TagSpecifications': ([SpotFleetTagSpecification], False),
        'UserData': (str, False),
        'WeightedCapacity': (positive_integer, False),
    }


class LaunchTemplateOverrides(AWSProperty):
    props = {
        'AvailabilityZone': (str, False),
        'InstanceType': (str, False),
        'SpotPrice': (str, False),
        'SubnetId': (str, False),
        'WeightedCapacity': (double, False)
    }


class LaunchTemplateConfigs(AWSProperty):
    props = {
        'LaunchTemplateSpecification': (LaunchTemplateSpecification, True),
        'Overrides': ([LaunchTemplateOverrides], False)
    }


class ClassicLoadBalancer(AWSProperty):
    props = {
        'Name': (str, True)
    }


class ClassicLoadBalancersConfig(AWSProperty):
    props = {
        'ClassicLoadBalancers': ([ClassicLoadBalancer], True)
    }


class TargetGroup(AWSProperty):
    props = {
        'Arn': (str, True)
    }


class TargetGroupConfig(AWSProperty):
    props = {
        'TargetGroups': ([TargetGroup], True),
    }


class LoadBalancersConfig(AWSProperty):
    props = {
        'ClassicLoadBalancersConfig': ([ClassicLoadBalancersConfig], False),
        'TargetGroupsConfig': (TargetGroupConfig, False)
    }


class SpotFleetRequestConfigData(AWSProperty):

    props = {
        'AllocationStrategy': (str, False),
        'ExcessCapacityTerminationPolicy': (str, False),
        'IamFleetRole': (str, True),
        'InstanceInterruptionBehavior': (str, False),
        'LaunchSpecifications': ([LaunchSpecifications], False),
        'LaunchTemplateConfigs': ([LaunchTemplateConfigs], False),
        'LoadBalancersConfig': (LoadBalancersConfig, False),
        'ReplaceUnhealthyInstances': (boolean, False),
        'SpotPrice': (str, False),
        'TargetCapacity': (positive_integer, True),
        'TerminateInstancesWithExpiration': (boolean, False),
        'Type': (str, False),
        'ValidFrom': (str, False),
        'ValidUntil': (str, False),
    }

    def validate(self):
        conds = [
            'LaunchSpecifications',
            'LaunchTemplateConfigs'
        ]
        exactly_one(self.__class__.__name__, self.properties, conds)


class SpotFleet(AWSObject):
    resource_type = "AWS::EC2::SpotFleet"

    props = {
        'SpotFleetRequestConfigData': (SpotFleetRequestConfigData, True),
    }


class PlacementGroup(AWSObject):
    resource_type = "AWS::EC2::PlacementGroup"

    props = {
        'Strategy': (str, True),
    }


class SubnetCidrBlock(AWSObject):
    resource_type = "AWS::EC2::SubnetCidrBlock"

    props = {
        'Ipv6CidrBlock': (str, True),
        'SubnetId': (str, True),
    }


class VPCCidrBlock(AWSObject):
    resource_type = "AWS::EC2::VPCCidrBlock"

    props = {
        'AmazonProvidedIpv6CidrBlock': (boolean, False),
        'CidrBlock': (str, False),
        'VpcId': (str, True),
    }


class TagSpecifications(AWSProperty):
    props = {
        'ResourceType': (str, False),
        'Tags': ((Tags, list), False)
    }


class SpotOptions(AWSProperty):
    props = {
        'BlockDurationMinutes': (integer, False),
        'InstanceInterruptionBehavior': (str, False),
        'MaxPrice': (str, False),
        'SpotInstanceType': (str, False),
        'ValidUntil': (str, False),
    }


class InstanceMarketOptions(AWSProperty):
    props = {
        'MarketType': (str, False),
        'SpotOptions': (SpotOptions, False)
    }


class LaunchTemplateCreditSpecification(AWSProperty):
    props = {
        'CpuCredits': (str, False),
    }


class LaunchTemplateData(AWSProperty):
    props = {
        'BlockDeviceMappings': ([BlockDeviceMapping], False),
        'CpuOptions': (CpuOptions, False),
        'CreditSpecification': (LaunchTemplateCreditSpecification, False),
        'DisableApiTermination': (boolean, False),
        'EbsOptimized': (boolean, False),
        'ElasticGpuSpecifications': ([ElasticGpuSpecification], False),
        'IamInstanceProfile': (IamInstanceProfile, False),
        'ImageId': (str, False),
        'InstanceInitiatedShutdownBehavior': (str, False),
        'InstanceMarketOptions': (InstanceMarketOptions, False),
        'InstanceType': (str, False),
        'KernelId': (str, False),
        'KeyName': (str, False),
        'LicenseSpecifications': ([LicenseSpecification], False),
        'Monitoring': (Monitoring, False),
        'NetworkInterfaces': ([NetworkInterfaces], False),
        'Placement': (Placement, False),
        'RamDiskId': (str, False),
        'SecurityGroups': (list, False),
        'SecurityGroupIds': (list, False),
        'TagSpecifications': ([TagSpecifications], False),
        'UserData': (str, False)
    }


class LaunchTemplate(AWSObject):
    resource_type = "AWS::EC2::LaunchTemplate"
    props = {
        'LaunchTemplateData': (LaunchTemplateData, False),
        'LaunchTemplateName': (str, False),
    }


class TrafficMirrorFilter(AWSObject):
    resource_type = "AWS::EC2::TrafficMirrorFilter"

    props = {
        'Description': (str, False),
        'NetworkServices': ([str], False),
        'Tags': (Tags, False),
    }


class TrafficMirrorPortRange(AWSProperty):
    props = {
        'FromPort': (integer, True),
        'ToPort': (integer, True),
    }


class TrafficMirrorFilterRule(AWSObject):
    resource_type = "AWS::EC2::TrafficMirrorFilterRule"

    props = {
        'Description': (str, False),
        'DestinationCidrBlock': (str, True),
        'DestinationPortRange': (TrafficMirrorPortRange, False),
        'Protocol': (integer, False),
        'RuleAction': (str, True),
        'RuleNumber': (integer, True),
        'SourceCidrBlock': (str, True),
        'SourcePortRange': (TrafficMirrorPortRange, False),
        'TrafficDirection': (str, True),
        'TrafficMirrorFilterId': (str, True),
    }


class TrafficMirrorSession(AWSObject):
    resource_type = "AWS::EC2::TrafficMirrorSession"

    props = {
        'Description': (str, False),
        'NetworkInterfaceId': (str, True),
        'PacketLength': (integer, False),
        'SessionNumber': (integer, True),
        'Tags': (Tags, False),
        'TrafficMirrorFilterId': (str, True),
        'TrafficMirrorTargetId': (str, True),
        'VirtualNetworkId': (integer, False),
    }


class TrafficMirrorTarget(AWSObject):
    resource_type = "AWS::EC2::TrafficMirrorTarget"

    props = {
        'Description': (str, False),
        'NetworkInterfaceId': (str, False),
        'NetworkLoadBalancerArn': (str, False),
        'Tags': (Tags, False),
    }


class TransitGateway(AWSObject):
    resource_type = "AWS::EC2::TransitGateway"
    props = {
        'AmazonSideAsn': (integer, False),
        'AutoAcceptSharedAttachments': (str, False),
        'DefaultRouteTableAssociation': (str, False),
        'DefaultRouteTablePropagation': (str, False),
        'DnsSupport': (str, False),
        'Tags': ((Tags, list), False),
        'VpnEcmpSupport': (str, False),
    }


class TransitGatewayAttachment(AWSObject):
    resource_type = "AWS::EC2::TransitGatewayAttachment"
    props = {
        'SubnetIds': ([str], True),
        'Tags': ((Tags, list), False),
        'TransitGatewayId': (str, True),
        'VpcId': (str, True),
    }


class TransitGatewayRoute(AWSObject):
    resource_type = "AWS::EC2::TransitGatewayRoute"
    props = {
        'Blackhole': (boolean, False),
        'DestinationCidrBlock': (str, False),
        'TransitGatewayAttachmentId': (str, False),
        'TransitGatewayRouteTableId': (str, True),
    }


class TransitGatewayRouteTable(AWSObject):
    resource_type = "AWS::EC2::TransitGatewayRouteTable"
    props = {
        'Tags': ((Tags, list), False),
        'TransitGatewayId': (str, True),
    }


class TransitGatewayRouteTableAssociation(AWSObject):
    resource_type = "AWS::EC2::TransitGatewayRouteTableAssociation"
    props = {
        'TransitGatewayAttachmentId': (str, True),
        'TransitGatewayRouteTableId': (str, True),
    }


class TransitGatewayRouteTablePropagation(AWSObject):
    resource_type = "AWS::EC2::TransitGatewayRouteTablePropagation"
    props = {
        'TransitGatewayAttachmentId': (str, True),
        'TransitGatewayRouteTableId': (str, True),
    }


class FleetLaunchTemplateSpecificationRequest(AWSProperty):
    props = {
        'LaunchTemplateId': (str, False),
        'LaunchTemplateName': (str, False),
        'Version': (str, False),
    }


class FleetLaunchTemplateOverridesRequest(AWSProperty):
    props = {
        'AvailabilityZone': (str, False),
        'InstanceType': (str, False),
        'MaxPrice': (str, False),
        'Priority': (double, False),
        'SubnetId': (str, False),
        'WeightedCapacity': (double, False),
    }


class FleetLaunchTemplateConfigRequest(AWSProperty):
    props = {
        'LaunchTemplateSpecification': (
            FleetLaunchTemplateSpecificationRequest,
            False
        ),
        'Overrides': ([FleetLaunchTemplateOverridesRequest], False),
    }


class OnDemandOptionsRequest(AWSProperty):
    props = {
        'AllocationStrategy': (str, False),
    }


class SpotOptionsRequest(AWSProperty):
    props = {
        'AllocationStrategy': (str, False),
        'InstanceInterruptionBehavior': (str, False),
        'InstancePoolsToUseCount': (integer, False),
    }


class TargetCapacitySpecificationRequest(AWSProperty):
    props = {
        'DefaultTargetCapacityType': (str, False),
        'OnDemandTargetCapacity': (integer, False),
        'SpotTargetCapacity': (integer, False),
        'TotalTargetCapacity': (integer, False),
    }


class EC2Fleet(AWSObject):
    resource_type = "AWS::EC2::EC2Fleet"
    props = {
        'ExcessCapacityTerminationPolicy': (str, False),
        'LaunchTemplateConfigs': ([FleetLaunchTemplateConfigRequest], True),
        'OnDemandOptions': (OnDemandOptionsRequest, False),
        'ReplaceUnhealthyInstances': (boolean, False),
        'SpotOptions': (SpotOptionsRequest, False),
        'TagSpecifications': ([TagSpecifications], False),
        'TargetCapacitySpecification': (TargetCapacitySpecificationRequest,
                                        False),
        'TerminateInstancesWithExpiration': (boolean, False),
        'Type': (str, False),
        'ValidFrom': (str, False),
        'ValidUntil': (str, False),
    }


class CapacityReservation(AWSObject):
    resource_type = "AWS::EC2::CapacityReservation"
    props = {
        'AvailabilityZone': (str, True),
        'EbsOptimized': (boolean, False),
        'EndDate': (str, False),
        'EndDateType': (str, False),
        'EphemeralStorage': (boolean, False),
        'InstanceCount': (integer, True),
        'InstanceMatchCriteria': (str, False),
        'InstancePlatform': (str, True),
        'InstanceType': (str, True),
        'TagSpecifications': ([TagSpecifications], False),
        'Tenancy': (str, False),
    }


class ClientVpnAuthorizationRule(AWSObject):
    resource_type = "AWS::EC2::ClientVpnAuthorizationRule"

    props = {
        'AccessGroupId': (str, False),
        'AuthorizeAllGroups': (boolean, False),
        'ClientVpnEndpointId': (str, True),
        'Description': (str, False),
        'TargetNetworkCidr': (str, True),
    }


class CertificateAuthenticationRequest(AWSProperty):
    props = {
        'ClientRootCertificateChainArn': (str, True),
    }


class DirectoryServiceAuthenticationRequest(AWSProperty):
    props = {
        'DirectoryId': (str, True),
    }


class ClientAuthenticationRequest(AWSProperty):
    props = {
        'ActiveDirectory': (DirectoryServiceAuthenticationRequest, False),
        'MutualAuthentication': (CertificateAuthenticationRequest, False),
        'Type': (str, True),
    }


class ConnectionLogOptions(AWSProperty):
    props = {
        'CloudwatchLogGroup': (str, False),
        'CloudwatchLogStream': (str, False),
        'Enabled': (boolean, True),
    }


class ClientVpnEndpoint(AWSObject):
    resource_type = "AWS::EC2::ClientVpnEndpoint"

    props = {
        'AuthenticationOptions': ([ClientAuthenticationRequest], True),
        'ClientCidrBlock': (str, True),
        'ConnectionLogOptions': (ConnectionLogOptions, True),
        'Description': (str, False),
        'DnsServers': ([str], False),
        'ServerCertificateArn': (str, True),
        'SplitTunnel': (boolean, False),
        'TagSpecifications': ([TagSpecifications], False),
        'TransportProtocol': (str, False),
    }


class ClientVpnRoute(AWSObject):
    resource_type = "AWS::EC2::ClientVpnRoute"

    props = {
        'ClientVpnEndpointId': (str, True),
        'Description': (str, False),
        'DestinationCidrBlock': (str, True),
        'TargetVpcSubnetId': (str, True),
    }


class ClientVpnTargetNetworkAssociation(AWSObject):
    resource_type = "AWS::EC2::ClientVpnTargetNetworkAssociation"

    props = {
        'ClientVpnEndpointId': (str, True),
        'SubnetId': (str, True),
    }



# Copyright (c) 2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

import re

from . import AWSHelperFn, AWSObject, AWSProperty, Tags
from .validators import (boolean, network_port, integer, positive_integer,
                         integer_range)

# Taken from:
# http://docs.aws.amazon.com/AmazonRDS/latest/APIReference/API_CreateDBInstance.html

VALID_STORAGE_TYPES = ('standard', 'gp2', 'io1')
VALID_DB_ENGINES = ('MySQL', 'mysql', 'oracle-se1', 'oracle-se2', 'oracle-se',
                    'oracle-ee', 'sqlserver-ee', 'sqlserver-se',
                    'sqlserver-ex', 'sqlserver-web', 'postgres', 'aurora',
                    'aurora-mysql', 'aurora-postgresql', 'mariadb')
VALID_DB_ENGINE_MODES = ('provisioned', 'serverless')
VALID_LICENSE_MODELS = ('license-included', 'bring-your-own-license',
                        'general-public-license', 'postgresql-license')
VALID_SCALING_CONFIGURATION_CAPACITIES = (1, 2, 4, 8, 16, 32, 64, 128, 256)


def validate_iops(iops):
    """DBInstance Iops validation rules."""

    iops = integer(iops)
    if int(iops) == 0:
        return iops
    if int(iops) < 1000:
        raise ValueError("DBInstance Iops, if set, must be greater than 1000.")
    return iops


def validate_storage_type(storage_type):
    """Validate StorageType for DBInstance"""

    if storage_type not in VALID_STORAGE_TYPES:
        raise ValueError("DBInstance StorageType must be one of: %s" %
                         ", ".join(VALID_STORAGE_TYPES))
    return storage_type


def validate_engine(engine):
    """Validate database Engine for DBInstance """

    if engine not in VALID_DB_ENGINES:
        raise ValueError("DBInstance Engine must be one of: %s" %
                         ", ".join(VALID_DB_ENGINES))
    return engine


def validate_engine_mode(engine_mode):
    """Validate database EngineMode for DBCluster"""

    if engine_mode not in VALID_DB_ENGINE_MODES:
        raise ValueError("DBCluster EngineMode must be one of: %s" %
                         ", ".join(VALID_DB_ENGINE_MODES))
    return engine_mode


def validate_license_model(license_model):
    """Validate LicenseModel for DBInstance"""

    if license_model not in VALID_LICENSE_MODELS:
        raise ValueError("DBInstance LicenseModel must be one of: %s" %
                         ", ".join(VALID_LICENSE_MODELS))
    return license_model


def validate_backup_window(window):
    """Validate PreferredBackupWindow for DBInstance"""

    hour = r'[01]?[0-9]|2[0-3]'
    minute = r'[0-5][0-9]'
    r = ("(?P<start_hour>%s):(?P<start_minute>%s)-"
         "(?P<end_hour>%s):(?P<end_minute>%s)") % (hour, minute, hour, minute)
    range_regex = re.compile(r)
    m = range_regex.match(window)
    if not m:
        raise ValueError("DBInstance PreferredBackupWindow must be in the "
                         "format: hh24:mi-hh24:mi")
    start_ts = (int(m.group('start_hour')) * 60) + int(m.group('start_minute'))
    end_ts = (int(m.group('end_hour')) * 60) + int(m.group('end_minute'))
    if abs(end_ts - start_ts) < 30:
        raise ValueError("DBInstance PreferredBackupWindow must be at least "
                         "30 minutes long.")
    return window


def validate_maintenance_window(window):
    """Validate PreferredMaintenanceWindow for DBInstance"""

    days = ("Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun")
    day_re = r'[A-Z]{1}[a-z]{2}'
    hour = r'[01]?[0-9]|2[0-3]'
    minute = r'[0-5][0-9]'
    r = ("(?P<start_day>%s):(?P<start_hour>%s):(?P<start_minute>%s)-"
         "(?P<end_day>%s):(?P<end_hour>%s):(?P<end_minute>%s)") % (day_re,
                                                                   hour,
                                                                   minute,
                                                                   day_re,
                                                                   hour,
                                                                   minute)
    range_regex = re.compile(r)
    m = range_regex.match(window)
    if not m:
        raise ValueError("DBInstance PreferredMaintenanceWindow must be in "
                         "the format: ddd:hh24:mi-ddd:hh24:mi")
    if m.group('start_day') not in days or m.group('end_day') not in days:
        raise ValueError("DBInstance PreferredMaintenanceWindow day part of "
                         "ranges must be one of: %s" % ", ".join(days))
    start_ts = (days.index(m.group('start_day')) * 24 * 60) + \
        (int(m.group('start_hour')) * 60) + int(m.group('start_minute'))
    end_ts = (days.index(m.group('end_day')) * 24 * 60) + \
        (int(m.group('end_hour')) * 60) + int(m.group('end_minute'))
    if abs(end_ts - start_ts) < 30:
        raise ValueError("DBInstance PreferredMaintenanceWindow must be at "
                         "least 30 minutes long.")
    return window


def validate_backup_retention_period(days):
    """Validate BackupRetentionPeriod for DBInstance"""

    days = positive_integer(days)
    if int(days) > 35:
        raise ValueError("DBInstance BackupRetentionPeriod cannot be larger "
                         "than 35 days.")
    return days


def validate_capacity(capacity):
    """Validate ScalingConfiguration capacity for serverless DBCluster"""

    if capacity not in VALID_SCALING_CONFIGURATION_CAPACITIES:
        raise ValueError(
            "ScalingConfiguration capacity must be one of: {}".format(
                ", ".join(map(
                    str,
                    VALID_SCALING_CONFIGURATION_CAPACITIES
                ))
            )
        )
    return capacity


class DBInstanceRole(AWSProperty):
    props = {
        'FeatureName': (str, True),
        'RoleArn': (str, True),
        'Status': (str, False),
    }


class ProcessorFeature(AWSProperty):
    props = {
        'Name': (str, False),
        'Value': (str, False),
    }


class DBInstance(AWSObject):
    resource_type = "AWS::RDS::DBInstance"

    props = {
        'AllocatedStorage': (positive_integer, False),
        'AllowMajorVersionUpgrade': (boolean, False),
        'AssociatedRoles': ([DBInstanceRole], False),
        'AutoMinorVersionUpgrade': (boolean, False),
        'AvailabilityZone': (str, False),
        'BackupRetentionPeriod': (validate_backup_retention_period, False),
        'CharacterSetName': (str, False),
        'CopyTagsToSnapshot': (boolean, False),
        'DBClusterIdentifier': (str, False),
        'DBInstanceClass': (str, True),
        'DBInstanceIdentifier': (str, False),
        'DBName': (str, False),
        'DBParameterGroupName': (str, False),
        'DBSecurityGroups': (list, False),
        'DBSnapshotIdentifier': (str, False),
        'DBSubnetGroupName': (str, False),
        'DeleteAutomatedBackups': (boolean, False),
        'DeletionProtection': (boolean, False),
        'Domain': (str, False),
        'DomainIAMRoleName': (str, False),
        'EnableCloudwatchLogsExports': ([str], False),
        'EnableIAMDatabaseAuthentication': (boolean, False),
        'EnablePerformanceInsights': (boolean, False),
        'Engine': (validate_engine, False),
        'EngineVersion': (str, False),
        'Iops': (validate_iops, False),
        'KmsKeyId': (str, False),
        'LicenseModel': (validate_license_model, False),
        'MasterUsername': (str, False),
        'MasterUserPassword': (str, False),
        'MonitoringInterval': (positive_integer, False),
        'MonitoringRoleArn': (str, False),
        'MultiAZ': (boolean, False),
        'OptionGroupName': (str, False),
        'PerformanceInsightsKMSKeyId': (str, False),
        'PerformanceInsightsRetentionPeriod': (positive_integer, False),
        'Port': (network_port, False),
        'PreferredBackupWindow': (validate_backup_window, False),
        'PreferredMaintenanceWindow': (str, False),
        'ProcessorFeatures': ([ProcessorFeature], False),
        'PromotionTier': (positive_integer, False),
        'PubliclyAccessible': (boolean, False),
        'SourceDBInstanceIdentifier': (str, False),
        'SourceRegion': (str, False),
        'StorageEncrypted': (boolean, False),
        'StorageType': (str, False),
        'Tags': ((Tags, list), False),
        'UseDefaultProcessorFeatures': (boolean, False),
        'Timezone': (str, False),
        'VPCSecurityGroups': ([str], False),
    }

    def validate(self):
        if 'DBSnapshotIdentifier' not in self.properties:
            if 'Engine' not in self.properties:
                raise ValueError(
                    'Resource Engine is required in type %s'
                    % self.resource_type)

        if 'SourceDBInstanceIdentifier' in self.properties:

            invalid_replica_properties = (
                'BackupRetentionPeriod', 'DBName', 'MasterUsername',
                'MasterUserPassword', 'PreferredBackupWindow', 'MultiAZ',
                'DBSnapshotIdentifier',
            )

            invalid_properties = [s for s in list(self.properties.keys()) if
                                  s in invalid_replica_properties]

            if invalid_properties:
                raise ValueError(
                    ('{0} properties can\'t be provided when '
                     'SourceDBInstanceIdentifier is present '
                     'AWS::RDS::DBInstance.'
                     ).format(', '.join(sorted(invalid_properties))))

        if ('DBSnapshotIdentifier' not in self.properties and
            'SourceDBInstanceIdentifier' not in self.properties) and \
            ('MasterUsername' not in self.properties or
             'MasterUserPassword' not in self.properties) and \
                ('DBClusterIdentifier' not in self.properties):
            raise ValueError(
                r"Either (MasterUsername and MasterUserPassword) or"
                r" DBSnapshotIdentifier are required in type "
                r"AWS::RDS::DBInstance."
            )

        if 'KmsKeyId' in self.properties and \
           'StorageEncrypted' not in self.properties:
            raise ValueError(
                'If KmsKeyId is provided, StorageEncrypted is required '
                'AWS::RDS::DBInstance.'
            )

        nonetype = type(None)
        avail_zone = self.properties.get('AvailabilityZone', None)
        multi_az = self.properties.get('MultiAZ', None)
        if not (isinstance(avail_zone, (AWSHelperFn, nonetype)) and
                isinstance(multi_az, (AWSHelperFn, nonetype))):
            if avail_zone and multi_az in [True, 1, '1', 'true', 'True']:
                raise ValueError("AvailabiltyZone cannot be set on "
                                 "DBInstance if MultiAZ is set to true.")

        storage_type = self.properties.get('StorageType', None)
        if storage_type and storage_type == 'io1' and \
                'Iops' not in self.properties:
            raise ValueError("Must specify Iops if using StorageType io1")

        allocated_storage = self.properties.get('AllocatedStorage')
        iops = self.properties.get('Iops', None)
        if iops and not isinstance(iops, AWSHelperFn):
            if not isinstance(allocated_storage, AWSHelperFn) and \
                    int(allocated_storage) < 100:
                raise ValueError("AllocatedStorage must be at least 100 when "
                                 "Iops is set.")
            if not isinstance(allocated_storage, AWSHelperFn) and not \
                    isinstance(iops, AWSHelperFn) and \
                    float(iops) / float(allocated_storage) > 50.0:
                raise ValueError("AllocatedStorage must be no less than "
                                 "1/50th the provisioned Iops")

        return True


class DBParameterGroup(AWSObject):
    resource_type = "AWS::RDS::DBParameterGroup"

    props = {
        'Description': (str, False),
        'Family': (str, False),
        'Parameters': (dict, False),
        'Tags': ((Tags, list), False),
    }


class DBSubnetGroup(AWSObject):
    resource_type = "AWS::RDS::DBSubnetGroup"

    props = {
        'DBSubnetGroupDescription': (str, True),
        'DBSubnetGroupName': (str, False),
        'SubnetIds': (list, True),
        'Tags': ((Tags, list), False),
    }


class RDSSecurityGroup(AWSProperty):
    props = {
        'CIDRIP': (str, False),
        'EC2SecurityGroupId': (str, False),
        'EC2SecurityGroupName': (str, False),
        'EC2SecurityGroupOwnerId': (str, False),
    }


class DBSecurityGroup(AWSObject):
    resource_type = "AWS::RDS::DBSecurityGroup"

    props = {
        'EC2VpcId': (str, False),
        'DBSecurityGroupIngress': (list, True),
        'GroupDescription': (str, True),
        'Tags': ((Tags, list), False),
    }


class DBSecurityGroupIngress(AWSObject):
    resource_type = "AWS::RDS::DBSecurityGroupIngress"

    props = {
        'CIDRIP': (str, False),
        'DBSecurityGroupName': (str, True),
        'EC2SecurityGroupId': (str, False),
        'EC2SecurityGroupName': (str, False),
        'EC2SecurityGroupOwnerId': (str, False),
    }


class EventSubscription(AWSObject):
    resource_type = "AWS::RDS::EventSubscription"

    props = {
        'Enabled': (boolean, False),
        'EventCategories': ([str], False),
        'SnsTopicArn': (str, True),
        'SourceIds': ([str], False),
        'SourceType': (str, False),
    }


class OptionSetting(AWSProperty):
    props = {
        'Name': (str, False),
        'Value': (str, False),
    }


class OptionConfiguration(AWSProperty):
    props = {
        'DBSecurityGroupMemberships': ([str], False),
        'OptionName': (str, True),
        'OptionSettings': ([OptionSetting], False),
        'OptionVersion': (str, False),
        'Port': (network_port, False),
        'VpcSecurityGroupMemberships': ([str], False),
    }


class OptionGroup(AWSObject):
    resource_type = "AWS::RDS::OptionGroup"

    props = {
        'EngineName': (str, True),
        'MajorEngineVersion': (str, True),
        'OptionGroupDescription': (str, True),
        'OptionConfigurations': ([OptionConfiguration], True),
        'Tags': ((Tags, list), False),
    }


class DBClusterParameterGroup(AWSObject):
    resource_type = "AWS::RDS::DBClusterParameterGroup"

    props = {
        'Description': (str, True),
        'Family': (str, True),
        'Parameters': (dict, False),
        'Tags': ((Tags, list), False),
    }


class DBClusterRole(AWSProperty):
    props = {
        'FeatureName': (str, False),
        'RoleArn': (str, True),
        'Status': (str, False),
    }


class ScalingConfiguration(AWSProperty):
    props = {
        'AutoPause': (boolean, False),
        'MaxCapacity': (validate_capacity, False),
        'MinCapacity': (validate_capacity, False),
        'SecondsUntilAutoPause': (positive_integer, False),
    }


class DBCluster(AWSObject):
    resource_type = "AWS::RDS::DBCluster"

    props = {
        'AssociatedRoles': ([DBClusterRole], False),
        'AvailabilityZones': ([str], False),
        'BacktrackWindow': (integer_range(0, 259200), False),
        'BackupRetentionPeriod': (validate_backup_retention_period, False),
        'DatabaseName': (str, False),
        'DBClusterIdentifier': (str, False),
        'DBClusterParameterGroupName': (str, False),
        'DBSubnetGroupName': (str, False),
        'DeletionProtection': (boolean, False),
        'EnableCloudwatchLogsExports': ([str], False),
        'EnableHttpEndpoint': (boolean, False),
        'EnableIAMDatabaseAuthentication': (boolean, False),
        'Engine': (validate_engine, True),
        'EngineMode': (validate_engine_mode, False),
        'EngineVersion': (str, False),
        'KmsKeyId': (str, False),
        'MasterUsername': (str, False),
        'MasterUserPassword': (str, False),
        'Port': (network_port, False),
        'PreferredBackupWindow': (validate_backup_window, False),
        'PreferredMaintenanceWindow': (str, False),
        'ReplicationSourceIdentifier': (str, False),
        'RestoreType': (str, False),
        'ScalingConfiguration': (ScalingConfiguration, False),
        'SnapshotIdentifier': (str, False),
        'SourceDBClusterIdentifier': (str, False),
        'SourceRegion': (str, False),
        'StorageEncrypted': (boolean, False),
        'Tags': ((Tags, list), False),
        'UseLatestRestorableTime': (boolean, False),
        'VpcSecurityGroupIds': ([str], False),
    }



# Copyright (c) 2012-2018, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import (boolean, double)


class LogSetup(AWSProperty):
    props = {
        'Enable': (bool, False),
        'Types': ([str], False)
        }

    def validate(self):
        types = set(self.properties.get('Types'))
        conditionals = ['api', 'audit', 'authenticator', 'controllerManager',
                        'scheduler']
        if not (types.issubset(conditionals)):
            raise ValueError(
                '%s must be one of: %s' % (', '.join(types),
                                           ', '.join(conditionals)))


class Logging(AWSProperty):
    props = {
        'ClusterLogging': ([LogSetup], False)
    }


class ResourcesVpcConfig(AWSProperty):
    props = {
        'SecurityGroupIds': ([str], False),
        'SubnetIds': ([str], True),
    }


class Cluster(AWSObject):
    resource_type = "AWS::EKS::Cluster"

    props = {
        'Name': (str, False),
        'Logging': (Logging, False),
        'ResourcesVpcConfig': (ResourcesVpcConfig, True),
        'RoleArn': (str, True),
        'Version': (str, False),
    }


class RemoteAccess(AWSProperty):
    props = {
        'Ec2SshKey': (str, True),
        'SourceSecurityGroups': ([str], False),
    }


class ScalingConfig(AWSProperty):
    props = {
        'DesiredSize': (double, False),
        'MaxSize': (double, False),
        'MinSize': (double, False),
    }


class Nodegroup(AWSObject):
    resource_type = "AWS::EKS::Nodegroup"

    props = {
        'AmiType': (str, False),
        'ClusterName': (str, True),
        'DiskSize': (double, False),
        'ForceUpdateEnabled': (boolean, False),
        'InstanceTypes': ([str], False),
        'Labels': (str, False),
        'NodegroupName': (str, False),
        'NodeRole': (str, True),
        'ReleaseVersion': (str, False),
        'RemoteAccess': (RemoteAccess, False),
        'ScalingConfig': (ScalingConfig, False),
        'Subnets': ([str], False),
        'Tags': (Tags, False),
        'Version': (str, False),
    }



# Copyright (c) 2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

import re

from . import AWSObject, AWSProperty, Tags
from .validators import boolean, integer, network_port


def validate_node_group_id(node_group_id):
    if re.match(r'\d{1,4}', node_group_id):
        return node_group_id
    raise ValueError("Invalid NodeGroupId: %s" % node_group_id)


class CacheCluster(AWSObject):
    resource_type = "AWS::ElastiCache::CacheCluster"

    props = {
        'AutoMinorVersionUpgrade': (boolean, False),
        'AZMode': (str, False),
        'CacheNodeType': (str, True),
        'CacheParameterGroupName': (str, False),
        'CacheSecurityGroupNames': ([str], False),
        'CacheSubnetGroupName': (str, False),
        'ClusterName': (str, False),
        'Engine': (str, True),
        'EngineVersion': (str, False),
        'NotificationTopicArn': (str, False),
        'NumCacheNodes': (integer, True),
        'Port': (integer, False),
        'PreferredAvailabilityZone': (str, False),
        'PreferredAvailabilityZones': ([str], False),
        'PreferredMaintenanceWindow': (str, False),
        'SnapshotArns': ([str], False),
        'SnapshotName': (str, False),
        'SnapshotRetentionLimit': (integer, False),
        'SnapshotWindow': (str, False),
        'Tags': (Tags, False),
        'VpcSecurityGroupIds': ([str], False),
    }

    def validate(self):
        # Check that AZMode is "cross-az" if more than one Availability zone
        # is specified in PreferredAvailabilityZones
        preferred_azs = self.properties.get('PreferredAvailabilityZones')
        if preferred_azs is not None and \
                isinstance(preferred_azs, list) and \
                len(preferred_azs) > 1:
            if self.properties.get('AZMode') != 'cross-az':
                raise ValueError('AZMode must be "cross-az" if more than one a'
                                 'vailability zone is specified in PreferredAv'
                                 'ailabilityZones: http://docs.aws.amazon.com/'
                                 'AWSCloudFormation/latest/UserGuide/aws-prope'
                                 'rties-elasticache-cache-cluster.html#cfn-ela'
                                 'sticache-cachecluster-azmode')

        return True


class ParameterGroup(AWSObject):
    resource_type = "AWS::ElastiCache::ParameterGroup"

    props = {
        'CacheParameterGroupFamily': (str, True),
        'Description': (str, True),
        'Properties': (dict, True),
    }


class SecurityGroup(AWSObject):
    resource_type = "AWS::ElastiCache::SecurityGroup"

    props = {
        'Description': (str, False),
    }


class SecurityGroupIngress(AWSObject):
    resource_type = "AWS::ElastiCache::SecurityGroupIngress"

    props = {
        'CacheSecurityGroupName': (str, True),
        'EC2SecurityGroupName': (str, True),
        'EC2SecurityGroupOwnerId': (str, False),
    }


class SubnetGroup(AWSObject):
    resource_type = "AWS::ElastiCache::SubnetGroup"

    props = {
        'CacheSubnetGroupName': (str, False),
        'Description': (str, True),
        'SubnetIds': (list, True),
    }


class ReplicationGroup(AWSObject):
    resource_type = "AWS::ElastiCache::ReplicationGroup"

    props = {
        'AtRestEncryptionEnabled': (boolean, False),
        'AuthToken': (str, False),
        'AutoMinorVersionUpgrade': (boolean, False),
        'AutomaticFailoverEnabled': (boolean, False),
        'CacheNodeType': (str, False),
        'CacheParameterGroupName': (str, False),
        'CacheSecurityGroupNames': ([str], False),
        'CacheSubnetGroupName': (str, False),
        'Engine': (str, False),
        'EngineVersion': (str, False),
        'KmsKeyId': (str, False),
        'NodeGroupConfiguration': (list, False),
        'NotificationTopicArn': (str, False),
        'NumCacheClusters': (integer, False),
        'NumNodeGroups': (integer, False),
        'Port': (network_port, False),
        'PreferredCacheClusterAZs': ([str], False),
        'PreferredMaintenanceWindow': (str, False),
        'PrimaryClusterId': (str, False),
        'ReplicasPerNodeGroup': (integer, False),
        'ReplicationGroupDescription': (str, True),
        'ReplicationGroupId': (str, False),
        'SecurityGroupIds': ([str], False),
        'SnapshotArns': ([str], False),
        'SnapshotName': (str, False),
        'SnapshotRetentionLimit': (integer, False),
        'SnapshottingClusterId': (str, False),
        'SnapshotWindow': (str, False),
        'Tags': (Tags, False),
        'TransitEncryptionEnabled': (boolean, False),
    }

    def validate(self):
        if 'NumCacheClusters' not in self.properties and \
           'NumNodeGroups' not in self.properties and \
           'ReplicasPerNodeGroup' not in self.properties and \
           'PrimaryClusterId' not in self.properties:
            raise ValueError(
                'One of PrimaryClusterId, NumCacheClusters, '
                'NumNodeGroups or ReplicasPerNodeGroup are required'
                'in type AWS::ElastiCache::ReplicationGroup'
                )

        return True


class NodeGroupConfiguration(AWSProperty):
    props = {
        'NodeGroupId': (validate_node_group_id, False),
        'PrimaryAvailabilityZone': (str, False),
        'ReplicaAvailabilityZones': ([str], False),
        'ReplicaCount': (integer, False),
        'Slots': (str, False),
    }



# Copyright (c) 2012-2017, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty
from .validators import integer


class InputParallelism(AWSProperty):
    props = {
        'Count': (integer, True),
    }


class RecordColumn(AWSProperty):
    props = {
        'Mapping': (str, False),
        'Name': (str, True),
        'SqlType': (str, True),
    }


class CSVMappingParameters(AWSProperty):
    props = {
        'RecordColumnDelimiter': (str, True),
        'RecordRowDelimiter': (str, True),
    }


class JSONMappingParameters(AWSProperty):
    props = {
        'RecordRowPath': (str, True),
    }


class MappingParameters(AWSProperty):
    props = {
        'CSVMappingParameters': (CSVMappingParameters, False),
        'JSONMappingParameters': (JSONMappingParameters, False),
    }


class RecordFormat(AWSProperty):
    props = {
        'MappingParameters': (MappingParameters, False),
        'RecordFormatType': (str, True),
    }


class InputSchema(AWSProperty):
    props = {
        'RecordColumns': ([RecordColumn], True),
        'RecordEncoding': (str, False),
        'RecordFormat': (RecordFormat, True),
    }


class KinesisFirehoseInput(AWSProperty):
    props = {
        'ResourceARN': (str, True),
        'RoleARN': (str, True),
    }


class KinesisStreamsInput(AWSProperty):
    props = {
        'ResourceARN': (str, True),
        'RoleARN': (str, True),
    }


class InputLambdaProcessor(AWSProperty):
    props = {
        'ResourceARN': (str, True),
        'RoleARN': (str, True),
    }


class InputProcessingConfiguration(AWSProperty):
    props = {
        'InputLambdaProcessor': (InputLambdaProcessor, False),
    }


class Input(AWSProperty):
    props = {
        'NamePrefix': (str, True),
        'InputParallelism': (InputParallelism, False),
        'InputSchema': (InputSchema, True),
        'KinesisFirehoseInput': (KinesisFirehoseInput, False),
        'KinesisStreamsInput': (KinesisStreamsInput, False),
        'InputProcessingConfiguration': (InputProcessingConfiguration, False),
    }


class Application(AWSObject):
    resource_type = "AWS::KinesisAnalytics::Application"

    props = {
        'ApplicationName': (str, False),
        'ApplicationDescription': (str, False),
        'ApplicationCode': (str, False),
        'Inputs': ([Input], True),
    }


class DestinationSchema(AWSProperty):
    props = {
        'RecordFormatType': (str, False),
    }


class KinesisFirehoseOutput(AWSProperty):
    props = {
        'ResourceARN': (str, True),
        'RoleARN': (str, True),
    }


class KinesisStreamsOutput(AWSProperty):
    props = {
        'ResourceARN': (str, True),
        'RoleARN': (str, True),
    }


class LambdaOutput(AWSProperty):
    props = {
        'ResourceARN': (str, True),
        'RoleARN': (str, True),
    }


class Output(AWSProperty):
    props = {
        'DestinationSchema': (DestinationSchema, True),
        'KinesisFirehoseOutput': (KinesisFirehoseOutput, False),
        'KinesisStreamsOutput': (KinesisStreamsOutput, False),
        'LambdaOutput': (LambdaOutput, False),
        'Name': (str, True),
    }


class ApplicationOutput(AWSObject):
    resource_type = "AWS::KinesisAnalytics::ApplicationOutput"

    props = {
        'ApplicationName': (str, True),
        'Output': (Output, True),
    }


class ReferenceSchema(AWSProperty):
    props = {
        'RecordColumns': ([RecordColumn], True),
        'RecordEncoding': (str, False),
        'RecordFormat': (RecordFormat, True),
    }


class S3ReferenceDataSource(AWSProperty):
    props = {
        'BucketARN': (str, False),
        'FileKey': (str, False),
        'ReferenceRoleARN': (str, False),
    }


class ReferenceDataSource(AWSProperty):
    props = {
        'ReferenceSchema': (ReferenceSchema, True),
        'S3ReferenceDataSource': (S3ReferenceDataSource, False),
        'TableName': (str, False),
    }


class ApplicationReferenceDataSource(AWSObject):
    resource_type = "AWS::KinesisAnalytics::ApplicationReferenceDataSource"

    props = {
        'ApplicationName': (str, True),
        'ReferenceDataSource': (ReferenceDataSource, True),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#

from . import AWSObject
from . import AWSProperty
from troposphere import Tags
from .validators import integer


class AudioLanguageSelection(AWSProperty):
    props = {
        'LanguageCode': (str, False),
        'LanguageSelectionPolicy': (str, False),
    }


class AudioPidSelection(AWSProperty):
    props = {
        'Pid': (integer, False),
    }


class AudioSelectorSettings(AWSProperty):
    props = {
        'AudioLanguageSelection': (AudioLanguageSelection, False),
        'AudioPidSelection': (AudioPidSelection, False),
    }


class AudioSelector(AWSProperty):
    props = {
        'Name': (str, False),
        'SelectorSettings': (AudioSelectorSettings, False),
    }


class AribSourceSettings(AWSProperty):
    props = {
    }


class DvbSubSourceSettings(AWSProperty):
    props = {
        'Pid': (integer, False),
    }


class EmbeddedSourceSettings(AWSProperty):
    props = {
        'Convert608To708': (str, False),
        'Scte20Detection': (str, False),
        'Source608ChannelNumber': (integer, False),
        'Source608TrackNumber': (integer, False),
    }


class Scte20SourceSettings(AWSProperty):
    props = {
        'Convert608To708': (str, False),
        'Source608ChannelNumber': (integer, False),
    }


class Scte27SourceSettings(AWSProperty):
    props = {
        'Pid': (integer, False),
    }


class TeletextSourceSettings(AWSProperty):
    props = {
        'PageNumber': (str, False),
    }


class CaptionSelectorSettings(AWSProperty):
    props = {
        'AribSourceSettings': (AribSourceSettings, False),
        'DvbSubSourceSettings': (DvbSubSourceSettings, False),
        'EmbeddedSourceSettings': (EmbeddedSourceSettings, False),
        'Scte20SourceSettings': (Scte20SourceSettings, False),
        'Scte27SourceSettings': (Scte27SourceSettings, False),
        'TeletextSourceSettings': (TeletextSourceSettings, False),
    }


class CaptionSelector(AWSProperty):
    props = {
        'LanguageCode': (str, False),
        'Name': (str, False),
        'SelectorSettings': (CaptionSelectorSettings, False),
    }


class HlsInputSettings(AWSProperty):
    props = {
        'Bandwidth': (integer, False),
        'BufferSegments': (integer, False),
        'Retries': (integer, False),
        'RetryInterval': (integer, False),
    }


class NetworkInputSettings(AWSProperty):
    props = {
        'HlsInputSettings': (HlsInputSettings, False),
        'ServerValidation': (str, False),
    }


class VideoSelectorPid(AWSProperty):
    props = {
        'Pid': (integer, False),
    }


class VideoSelectorProgramId(AWSProperty):
    props = {
        'ProgramId': (integer, False),
    }


class VideoSelectorSettings(AWSProperty):
    props = {
        'VideoSelectorPid': (VideoSelectorPid, False),
        'VideoSelectorProgramId': (VideoSelectorProgramId, False),
    }


class VideoSelector(AWSProperty):
    props = {
        'ColorSpace': (str, False),
        'ColorSpaceUsage': (str, False),
        'SelectorSettings': (VideoSelectorSettings, False),
    }


class InputSettings(AWSProperty):
    props = {
        'AudioSelectors': ([AudioSelector], False),
        'CaptionSelectors': ([CaptionSelector], False),
        'DeblockFilter': (str, False),
        'DenoiseFilter': (str, False),
        'FilterStrength': (integer, False),
        'InputFilter': (str, False),
        'NetworkInputSettings': (NetworkInputSettings, False),
        'SourceEndBehavior': (str, False),
        'VideoSelector': (VideoSelector, False),
    }


class InputAttachment(AWSProperty):
    props = {
        'InputAttachmentName': (str, False),
        'InputId': (str, False),
        'InputSettings': (InputSettings, False),
    }


class InputSpecification(AWSProperty):
    props = {
        'Codec': (str, False),
        'MaximumBitrate': (str, False),
        'Resolution': (str, False),
    }


class MediaPackageOutputDestinationSettings(AWSProperty):
    props = {
        'ChannelId': (str, False),
    }


class OutputDestinationSettings(AWSProperty):
    props = {
        'PasswordParam': (str, False),
        'StreamName': (str, False),
        'Url': (str, False),
        'Username': (str, False),
    }


class OutputDestination(AWSProperty):
    props = {
        'Id': (str, False),
        'MediaPackageSettings':
            ([MediaPackageOutputDestinationSettings], False),
        'Settings': ([OutputDestinationSettings], False),
    }


class Channel(AWSObject):
    resource_type = "AWS::MediaLive::Channel"

    props = {
        'ChannelClass': (str, False),
        'Destinations': ([OutputDestination], False),
        'EncoderSettings': (dict, False),
        'InputAttachments': ([InputAttachment], False),
        'InputSpecification': (InputSpecification, False),
        'LogLevel': (str, False),
        'Name': (str, False),
        'RoleArn': (str, False),
        'Tags': (Tags, False),
    }


class InputDestinationRequest(AWSProperty):
    props = {
        'StreamName': (str, False),
    }


class InputSourceRequest(AWSProperty):
    props = {
        'PasswordParam': (str, False),
        'Url': (str, False),
        'Username': (str, False),
    }


class InputVpcRequest(AWSProperty):
    props = {
        'SecurityGroupIds': ([str], False),
        'SubnetIds': ([str], False),
    }


class MediaConnectFlowRequest(AWSProperty):
    props = {
        'FlowArn': (str, False),
    }


class Input(AWSObject):
    resource_type = "AWS::MediaLive::Input"

    props = {
        'Destinations': ([InputDestinationRequest], False),
        'InputSecurityGroups': ([str], False),
        'MediaConnectFlows': ([MediaConnectFlowRequest], False),
        'Name': (str, False),
        'RoleArn': (str, False),
        'Sources': ([InputSourceRequest], False),
        'Tags': (Tags, False),
        'Type': (str, False),
        'Vpc': (InputVpcRequest, False),
    }


class InputWhitelistRuleCidr(AWSProperty):
    props = {
        'Cidr': (str, False),
    }


class InputSecurityGroup(AWSObject):
    resource_type = "AWS::MediaLive::InputSecurityGroup"

    props = {
        'Tags': (Tags, False),
        'WhitelistRules': ([InputWhitelistRuleCidr], False),
    }



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSHelperFn, AWSObject, AWSProperty, Tags
from .compat import policytypes
from .validators import integer


class RedrivePolicy(AWSProperty):
    props = {
        'deadLetterTargetArn': (str, False),
        'maxReceiveCount': (integer, False),
    }


class Queue(AWSObject):
    resource_type = "AWS::SQS::Queue"

    props = {
        'ContentBasedDeduplication': (bool, False),
        'DelaySeconds': (integer, False),
        'FifoQueue': (bool, False),
        'KmsMasterKeyId': (str, False),
        'KmsDataKeyReusePeriodSeconds': (integer, False),
        'MaximumMessageSize': (integer, False),
        'MessageRetentionPeriod': (integer, False),
        'QueueName': (str, False),
        'ReceiveMessageWaitTimeSeconds': (integer, False),
        'RedrivePolicy': (RedrivePolicy, False),
        'Tags': (Tags, False),
        'VisibilityTimeout': (integer, False),
    }

    def validate(self):
        if self.properties.get('FifoQueue'):
            queuename = self.properties.get('QueueName')
            if queuename is None or isinstance(queuename, AWSHelperFn):
                pass
            elif not queuename.endswith('.fifo'):
                raise ValueError("SQS: FIFO queues need to provide a "
                                 "QueueName that ends with '.fifo'")


class QueuePolicy(AWSObject):
    resource_type = "AWS::SQS::QueuePolicy"

    props = {
        'PolicyDocument': (policytypes, False),
        'Queues': (list, True),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 8.1.0


from . import AWSObject
from . import AWSProperty
from .validators import integer


class EventBus(AWSObject):
    resource_type = "AWS::Events::EventBus"

    props = {
        'EventSourceName': (str, False),
        'Name': (str, True),
    }


class Condition(AWSProperty):
    props = {
        'Key': (str, False),
        'Type': (str, False),
        'Value': (str, False),
    }


class EventBusPolicy(AWSObject):
    resource_type = "AWS::Events::EventBusPolicy"

    props = {
        'Action': (str, True),
        'Condition': (Condition, False),
        'EventBusName': (str, False),
        'Principal': (str, True),
        'StatementId': (str, True),
    }


class BatchArrayProperties(AWSProperty):
    props = {
        'Size': (integer, False),
    }


class BatchRetryStrategy(AWSProperty):
    props = {
        'Attempts': (integer, False),
    }


class BatchParameters(AWSProperty):
    props = {
        'ArrayProperties': (BatchArrayProperties, False),
        'JobDefinition': (str, True),
        'JobName': (str, True),
        'RetryStrategy': (BatchRetryStrategy, False),
    }


class AwsVpcConfiguration(AWSProperty):
    props = {
        'AssignPublicIp': (str, False),
        'SecurityGroups': ([str], False),
        'Subnets': ([str], True),
    }


class NetworkConfiguration(AWSProperty):
    props = {
        'AwsVpcConfiguration': (AwsVpcConfiguration, False),
    }


class EcsParameters(AWSProperty):
    props = {
        'Group': (str, False),
        'LaunchType': (str, False),
        'NetworkConfiguration': (NetworkConfiguration, False),
        'PlatformVersion': (str, False),
        'TaskCount': (integer, False),
        'TaskDefinitionArn': (str, True),
    }


class InputTransformer(AWSProperty):
    props = {
        'InputPathsMap': (dict, False),
        'InputTemplate': (str, True),
    }


class KinesisParameters(AWSProperty):
    props = {
        'PartitionKeyPath': (str, True),
    }


class RunCommandTarget(AWSProperty):
    props = {
        'Key': (str, True),
        'Values': ([str], True),
    }


class RunCommandParameters(AWSProperty):
    props = {
        'RunCommandTargets': ([RunCommandTarget], True),
    }


class SqsParameters(AWSProperty):
    props = {
        'MessageGroupId': (str, True),
    }


class Target(AWSProperty):
    props = {
        'Arn': (str, True),
        'BatchParameters': (BatchParameters, False),
        'EcsParameters': (EcsParameters, False),
        'Id': (str, True),
        'Input': (str, False),
        'InputPath': (str, False),
        'InputTransformer': (InputTransformer, False),
        'KinesisParameters': (KinesisParameters, False),
        'RoleArn': (str, False),
        'RunCommandParameters': (RunCommandParameters, False),
        'SqsParameters': (SqsParameters, False),
    }


class Rule(AWSObject):
    resource_type = "AWS::Events::Rule"

    props = {
        'Description': (str, False),
        'EventBusName': (str, False),
        'EventPattern': (dict, False),
        'Name': (str, False),
        'RoleArn': (str, False),
        'ScheduleExpression': (str, False),
        'State': (str, False),
        'Targets': ([Target], False),
    }



# Copyright (c) 2012-2018, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import boolean, integer


class AcceptedPortfolioShare(AWSObject):
    resource_type = "AWS::ServiceCatalog::AcceptedPortfolioShare"

    props = {
        'AcceptLanguage': (str, False),
        'PortfolioId': (str, True),
    }


class ProvisioningArtifactProperties(AWSProperty):
    props = {
        'Description': (str, False),
        'DisableTemplateValidation': (boolean, False),
        'Info': (dict, True),
        'Name': (str, False),
    }


class CloudFormationProduct(AWSObject):
    resource_type = "AWS::ServiceCatalog::CloudFormationProduct"

    props = {
        'AcceptLanguage': (str, False),
        'Description': (str, False),
        'Distributor': (str, False),
        'Name': (str, True),
        'Owner': (str, True),
        'ProvisioningArtifactParameters':
            ([ProvisioningArtifactProperties], True),
        'SupportDescription': (str, False),
        'SupportEmail': (str, False),
        'SupportUrl': (str, False),
        'Tags': (Tags, False),
    }


class ProvisioningParameter(AWSProperty):
    props = {
        'Key': (str, False),
        'Value': (str, False),
    }


class ProvisioningPreferences(AWSProperty):
    props = {
        'StackSetAccounts': ([str], False),
        'StackSetFailureToleranceCount': (integer, False),
        'StackSetFailureTolerancePercentage': (integer, False),
        'StackSetMaxConcurrencyCount': (integer, False),
        'StackSetMaxConcurrencyPercentage': (integer, False),
        'StackSetOperationType': (str, False),
        'StackSetRegions': ([str], False),
    }


class CloudFormationProvisionedProduct(AWSObject):
    resource_type = "AWS::ServiceCatalog::CloudFormationProvisionedProduct"

    props = {
        'AcceptLanguage': (str, False),
        'NotificationArns': ([str], False),
        'PathId': (str, False),
        'ProductId': (str, False),
        'ProductName': (str, False),
        'ProvisionedProductName': (str, False),
        'ProvisioningArtifactId': (str, False),
        'ProvisioningArtifactName': (str, False),
        'ProvisioningParameters': ([ProvisioningParameter], False),
        'ProvisioningPreferences': (ProvisioningPreferences, False),
        'Tags': (Tags, False),
    }


class LaunchNotificationConstraint(AWSObject):
    resource_type = "AWS::ServiceCatalog::LaunchNotificationConstraint"

    props = {
        'AcceptLanguage': (str, False),
        'Description': (str, False),
        'NotificationArns': ([str], True),
        'PortfolioId': (str, True),
        'ProductId': (str, True),
    }


class LaunchRoleConstraint(AWSObject):
    resource_type = "AWS::ServiceCatalog::LaunchRoleConstraint"

    props = {
        'AcceptLanguage': (str, False),
        'Description': (str, False),
        'PortfolioId': (str, True),
        'ProductId': (str, True),
        'RoleArn': (str, True),
    }


class LaunchTemplateConstraint(AWSObject):
    resource_type = "AWS::ServiceCatalog::LaunchTemplateConstraint"

    props = {
        'AcceptLanguage': (str, False),
        'Description': (str, False),
        'PortfolioId': (str, True),
        'ProductId': (str, True),
        'Rules': (str, True),
    }


class Portfolio(AWSObject):
    resource_type = "AWS::ServiceCatalog::Portfolio"

    props = {
        'AcceptLanguage': (str, False),
        'Description': (str, False),
        'DisplayName': (str, True),
        'ProviderName': (str, True),
        'Tags': (Tags, False),
    }


class PortfolioPrincipalAssociation(AWSObject):
    resource_type = "AWS::ServiceCatalog::PortfolioPrincipalAssociation"

    props = {
        'AcceptLanguage': (str, False),
        'PortfolioId': (str, True),
        'PrincipalARN': (str, True),
        'PrincipalType': (str, True),
    }


class PortfolioProductAssociation(AWSObject):
    resource_type = "AWS::ServiceCatalog::PortfolioProductAssociation"

    props = {
        'AcceptLanguage': (str, False),
        'PortfolioId': (str, True),
        'ProductId': (str, True),
        'SourcePortfolioId': (str, False),
    }


class PortfolioShare(AWSObject):
    resource_type = "AWS::ServiceCatalog::PortfolioShare"

    props = {
        'AcceptLanguage': (str, False),
        'AccountId': (str, True),
        'PortfolioId': (str, True),
    }


def validate_tag_update(update):
    valid_tag_update_values = [
        "ALLOWED",
        "NOT_ALLOWED",
    ]
    if update not in valid_tag_update_values:
        raise ValueError(
            "{} is not a valid tag update value".format(update)
        )
    return update


class ResourceUpdateConstraint(AWSObject):
    resource_type = "AWS::ServiceCatalog::ResourceUpdateConstraint"

    props = {
        'AcceptLanguage': (str, False),
        'Description': (str, False),
        'PortfolioId': (str, True),
        'ProductId': (str, True),
        'TagUpdateOnProvisionedProduct': (validate_tag_update, True),
    }


class StackSetConstraint(AWSObject):
    resource_type = "AWS::ServiceCatalog::StackSetConstraint"

    props = {
        'AcceptLanguage': (str, False),
        'AccountList': ([str], True),
        'AdminRole': (str, True),
        'Description': (str, True),
        'ExecutionRole': (str, True),
        'PortfolioId': (str, True),
        'ProductId': (str, True),
        'RegionList': ([str], True),
        'StackInstanceControl': (str, True),
    }


class TagOption(AWSObject):
    resource_type = "AWS::ServiceCatalog::TagOption"

    props = {
        'Active': (boolean, False),
        'Key': (str, True),
        'Value': (str, True),
    }


class TagOptionAssociation(AWSObject):
    resource_type = "AWS::ServiceCatalog::TagOptionAssociation"

    props = {
        'ResourceId': (str, True),
        'TagOptionId': (str, True),
    }



# Copyright (c) 2015, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .compat import policytypes
from .validators import integer, boolean

VALID_TARGET_TYPES = ('AWS::RDS::DBInstance', 'AWS::RDS::DBCluster')


def validate_target_types(target_type):
    """Target types validation rule."""

    if target_type not in VALID_TARGET_TYPES:
        raise ValueError("Target type must be one of : %s" %
                         ", ".join(VALID_TARGET_TYPES))
    return target_type


class ResourcePolicy(AWSObject):
    resource_type = "AWS::SecretsManager::ResourcePolicy"

    props = {
        'SecretId': (str, True),
        'ResourcePolicy': (policytypes, True),
    }


class RotationRules(AWSProperty):
    props = {
        'AutomaticallyAfterDays': (integer, False),
    }


class RotationSchedule(AWSObject):
    resource_type = "AWS::SecretsManager::RotationSchedule"

    props = {
        'SecretId': (str, True),
        'RotationLambdaARN': (str, True),
        'RotationRules': (RotationRules, False)
    }


class SecretTargetAttachment(AWSObject):
    resource_type = "AWS::SecretsManager::SecretTargetAttachment"

    props = {
        'SecretId': (str, True),
        'TargetId': (str, True),
        'TargetType': (validate_target_types, True),
    }


class GenerateSecretString(AWSProperty):
    props = {
        'ExcludeUppercase': (boolean, False),
        'RequireEachIncludedType': (boolean, False),
        'IncludeSpace': (boolean, False),
        'ExcludeCharacters': (str, False),
        'GenerateStringKey': (str, False),
        'PasswordLength': (integer, False),
        'ExcludePunctuation': (boolean, False),
        'ExcludeLowercase': (boolean, False),
        'SecretStringTemplate': (str, False),
        'ExcludeNumbers': (boolean, False),
    }


class Secret(AWSObject):
    resource_type = "AWS::SecretsManager::Secret"

    props = {
        'Description': (str, False),
        'KmsKeyId': (str, False),
        'SecretString': (str, False),
        'GenerateSecretString': (GenerateSecretString, False),
        'Name': (str, False),
        'Tags': ((Tags, list), False),
    }



# Copyright (c) 2014, Guillem Anguera <ganguera@gmail.com>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import integer


class StreamEncryption(AWSProperty):
    props = {
        'EncryptionType': (str, True),
        'KeyId': (str, True),
    }


class Stream(AWSObject):
    resource_type = "AWS::Kinesis::Stream"

    props = {
        'Name': (str, False),
        'RetentionPeriodHours': (integer, False),
        'ShardCount': (integer, False),
        'StreamEncryption': (StreamEncryption, False),
        'Tags': ((Tags, list), False),
    }


class StreamConsumer(AWSObject):
    resource_type = "AWS::Kinesis::StreamConsumer"

    props = {
        'ConsumerName': (str, True),
        'StreamARN': (str, True),
    }



# Copyright (c) 2015, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty
from .validators import boolean


class VpcSettings(AWSProperty):
    props = {
        'SubnetIds': ([str], True),
        'VpcId': (str, True),
    }


class MicrosoftAD(AWSObject):
    resource_type = "AWS::DirectoryService::MicrosoftAD"

    props = {
        'CreateAlias': (boolean, False),
        'Edition': (str, False),
        'EnableSso': (boolean, False),
        'Name': (str, True),
        'Password': (str, True),
        'ShortName': (str, False),
        'VpcSettings': (VpcSettings, True)
    }


class SimpleAD(AWSObject):
    resource_type = "AWS::DirectoryService::SimpleAD"

    props = {
        'CreateAlias': (boolean, False),
        'Description': (str, False),
        'EnableSso': (boolean, False),
        'Name': (str, True),
        'Password': (str, True),
        'ShortName': (str, False),
        'Size': (str, True),
        'VpcSettings': (VpcSettings, True),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 10.0.0


from . import AWSObject
from troposphere import Tags


class Discoverer(AWSObject):
    resource_type = "AWS::EventSchemas::Discoverer"

    props = {
        'Description': (str, False),
        'SourceArn': (str, True),
        'Tags': (Tags, False),
    }


class Registry(AWSObject):
    resource_type = "AWS::EventSchemas::Registry"

    props = {
        'Description': (str, False),
        'RegistryName': (str, False),
        'Tags': (Tags, False),
    }


class Schema(AWSObject):
    resource_type = "AWS::EventSchemas::Schema"

    props = {
        'Content': (str, True),
        'Description': (str, False),
        'RegistryName': (str, True),
        'SchemaName': (str, False),
        'Tags': (Tags, False),
        'Type': (str, True),
    }



# Copyright (c) 2014, Guillem Anguera <ganguera@gmail.com>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty
from .validators import integer, boolean, json_checker


VALID_RUNTIME_ENVIRONMENTS = ('SQL-1.0', 'FLINK-1_6')


def validate_runtime_environment(runtime_environment):
    """Validate RuntimeEnvironment for Application"""

    if runtime_environment not in VALID_RUNTIME_ENVIRONMENTS:
        raise ValueError("Application RuntimeEnvironment must be one of: %s" %
                         ", ".join(VALID_RUNTIME_ENVIRONMENTS))
    return runtime_environment


class S3ContentLocation(AWSProperty):
    props = {
        'BucketARN': (str, False),
        'FileKey': (str, False),
        'ObjectVersion': (str, False)
    }


class CodeContent(AWSProperty):
    props = {
        'ZipFileContent': (str, False),
        'S3ContentLocation': (S3ContentLocation, False),
        'TextContent': (str, False),
    }


class ApplicationCodeConfiguration(AWSProperty):
    props = {
        'CodeContentType': (str, True),
        'CodeContent': (CodeContent, True),
    }


class PropertyGroup(AWSProperty):
    props = {
        'PropertyMap': (json_checker, False),
        'PropertyGroupId': (str, False),
    }


class EnvironmentProperties(AWSProperty):
    props = {
        'PropertyGroups': ([PropertyGroup], False),
    }


class CheckpointConfiguration(AWSProperty):
    props = {
        'ConfigurationType': (str, True),
        'CheckpointInterval': (integer, False),
        'MinPauseBetweenCheckpoints': (integer, False),
        'CheckpointingEnabled': (boolean, False)
    }


class MonitoringConfiguration(AWSProperty):
    props = {
        'ConfigurationType': (str, True),
        'MetricsLevel': (str, False),
        'LogLevel': (str, False),
    }


class ParallelismConfiguration(AWSProperty):
    props = {
        'ConfigurationType': (str, True),
        'ParallelismPerKPU': (integer, False),
        'AutoScalingEnabled': (boolean, False),
        'Parallelism': (integer, False),
    }


class FlinkApplicationConfiguration(AWSProperty):
    props = {
        'CheckpointConfiguration': (CheckpointConfiguration, False),
        'ParallelismConfiguration': (ParallelismConfiguration, False),
        'MonitoringConfiguration': (MonitoringConfiguration, False),
    }


class InputParallelism(AWSProperty):
    props = {
        'Count': (integer, False),
    }


class InputLambdaProcessor(AWSProperty):
    props = {
        'ResourceARN': (str, True),
    }


class InputProcessingConfiguration(AWSProperty):
    props = {
        'InputLambdaProcessor': (InputLambdaProcessor, False),
    }


class RecordColumn(AWSProperty):
    props = {
        'Mapping': (str, False),
        'SqlType': (str, True),
        'Name': (str, True),
    }


class JSONMappingParameters(AWSProperty):
    props = {
        'RecordRowPath': (str, True),
    }


class CSVMappingParameters(AWSProperty):
    props = {
        'RecordRowDelimiter': (str, True),
        'RecordColumnDelimiter': (str, True),
    }


class MappingParameters(AWSProperty):
    props = {
        'JSONMappingParameters': (JSONMappingParameters, False),
        'CSVMappingParameters': (CSVMappingParameters, False),
    }


class RecordFormat(AWSProperty):
    props = {
        'MappingParameters': (MappingParameters, False),
        'RecordFormatType': (str, True),
    }


class InputSchema(AWSProperty):
    props = {
        'RecordEncoding': (str, False),
        'RecordColumns': ([RecordColumn], True),
        'RecordFormat': (RecordFormat, True),
    }


class KinesisStreamsInput(AWSProperty):
    props = {
        'ResourceARN': (str, True),
    }


class KinesisFirehoseInput(AWSProperty):
    props = {
        'ResourceARN': (str, True),
    }


class Input(AWSProperty):
    props = {
        'NamePrefix': (str, False),
        'InputSchema': (InputSchema, True),
        'KinesisStreamsInput': (KinesisStreamsInput, False),
        'KinesisFirehoseInput': (KinesisFirehoseInput, False),
        'InputProcessingConfiguration': (InputProcessingConfiguration, False),
        'InputParallelism': (InputParallelism, False),
    }


class SqlApplicationConfiguration(AWSProperty):
    props = {
        'Inputs': ([Input], False),
    }


class ApplicationSnapshotConfiguration(AWSProperty):
    props = {
        'SnapshotsEnabled': (boolean, True),
    }


class ApplicationConfiguration(AWSProperty):
    props = {
        'ApplicationCodeConfiguration': (ApplicationCodeConfiguration, False),
        'EnvironmentProperties': (EnvironmentProperties, False),
        'FlinkApplicationConfiguration': (FlinkApplicationConfiguration, False),  # NOQA
        'SqlApplicationConfiguration': (SqlApplicationConfiguration, False),
        'ApplicationSnapshotConfiguration': (ApplicationSnapshotConfiguration, False),  # NOQA
    }


class Application(AWSObject):
    resource_type = "AWS::KinesisAnalyticsV2::Application"

    props = {
        'ApplicationName': (str, False),
        'RuntimeEnvironment': (validate_runtime_environment, True),
        'ApplicationConfiguration': (ApplicationConfiguration, False),
        'ApplicationDescription': (str, False),
        'ServiceExecutionRole': (str, True),
    }


class S3ReferenceDataSource(AWSProperty):
    props = {
        'BucketARN': (str, False),
        'FileKey': (str, False),
    }


class ReferenceSchema(AWSProperty):
    props = {
        'RecordEncoding': (str, False),
        'RecordColumns': ([RecordColumn], True),
        'RecordFormat': (RecordFormat, True),
    }


class ReferenceDataSource(AWSProperty):
    props = {
        'ReferenceSchema': (ReferenceSchema, True),
        'TableName': (str, False),
        'S3ReferenceDataSource': (S3ReferenceDataSource, False),
    }


class ApplicationReferenceDataSource(AWSObject):
    resource_type = "AWS::KinesisAnalyticsV2::ApplicationReferenceDataSource"

    props = {
        'ApplicationName': (str, True),
        'ReferenceDataSource': (ReferenceDataSource, True),
    }


class LambdaOutput(AWSProperty):
    props = {
        'ResourceARN': (str, True),
    }


class KinesisFirehoseOutput(AWSProperty):
    props = {
        'ResourceARN': (str, True),
    }


class KinesisStreamsOutput(AWSProperty):
    props = {
        'ResourceARN': (str, True),
    }


class DestinationSchema(AWSProperty):
    props = {
        'RecordFormatType': (str, False),
    }


class Output(AWSProperty):
    props = {
        "DestinationSchema": (DestinationSchema, True),
        "LambdaOutput": (LambdaOutput, False),
        "KinesisFirehoseOutput": (KinesisFirehoseOutput, False),
        "KinesisStreamsOutput": (KinesisStreamsOutput, False),
        "Name": (str, False),
    }


class ApplicationOutput(AWSObject):
    resource_type = "AWS::KinesisAnalyticsV2::ApplicationOutput"

    props = {
        'ApplicationName': (str, True),
        'Output': (Output, True),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


from . import AWSObject
from . import AWSProperty
from .validators import boolean
from .validators import integer


class EBSStorageInfo(AWSProperty):
    props = {
        'VolumeSize': (integer, False),
    }


class StorageInfo(AWSProperty):
    props = {
        'EBSStorageInfo': (EBSStorageInfo, False),
    }


class BrokerNodeGroupInfo(AWSProperty):
    props = {
        'BrokerAZDistribution': (str, False),
        'ClientSubnets': ([str], True),
        'InstanceType': (str, True),
        'SecurityGroups': ([str], False),
        'StorageInfo': (StorageInfo, False),
    }


class Tls(AWSProperty):
    props = {
        'CertificateAuthorityArnList': ([str], False),
    }


class ClientAuthentication(AWSProperty):
    props = {
        'Tls': (Tls, False),
    }


class ConfigurationInfo(AWSProperty):
    props = {
        'Arn': (str, True),
        'Revision': (integer, True),
    }


class EncryptionAtRest(AWSProperty):
    props = {
        'DataVolumeKMSKeyId': (str, True),
    }


class EncryptionInTransit(AWSProperty):
    props = {
        'ClientBroker': (str, False),
        'InCluster': (boolean, False),
    }


class EncryptionInfo(AWSProperty):
    props = {
        'EncryptionAtRest': (EncryptionAtRest, False),
        'EncryptionInTransit': (EncryptionInTransit, False),
    }


class Cluster(AWSObject):
    resource_type = "AWS::MSK::Cluster"

    props = {
        'BrokerNodeGroupInfo': (BrokerNodeGroupInfo, True),
        'ClientAuthentication': (ClientAuthentication, False),
        'ClusterName': (str, True),
        'ConfigurationInfo': (ConfigurationInfo, False),
        'EncryptionInfo': (EncryptionInfo, False),
        'EnhancedMonitoring': (str, False),
        'KafkaVersion': (str, True),
        'NumberOfBrokerNodes': (integer, True),
        'Tags': (dict, False),
    }



from . import AWSObject, AWSProperty, Tags
from .validators import boolean

Bursting = 'bursting'
Provisioned = 'provisioned'


def throughput_mode_validator(mode):
    valid_modes = [Bursting, Provisioned]
    if mode not in valid_modes:
        raise ValueError(
            'ThroughputMode must be one of: "%s"' % (', '.join(valid_modes))
        )
    return mode


def provisioned_throughput_validator(throughput):
    if throughput < 0.0:
        raise ValueError(
            'ProvisionedThroughputInMibps must be greater than or equal to 0.0'
        )
    return throughput


class LifecyclePolicy(AWSProperty):
    props = {
        'TransitionToIA': (str, True),
    }


class FileSystem(AWSObject):
    resource_type = "AWS::EFS::FileSystem"

    props = {
        'Encrypted': (boolean, False),
        'FileSystemTags': (Tags, False),
        'KmsKeyId': (str, False),
        'LifecyclePolicies': ([LifecyclePolicy], False),
        'PerformanceMode': (str, False),
        'ProvisionedThroughputInMibps': (float, False),
        'ThroughputMode': (throughput_mode_validator, False),
    }


class MountTarget(AWSObject):
    resource_type = "AWS::EFS::MountTarget"

    props = {
        'FileSystemId': (str, True),
        'IpAddress': (str, False),
        'SecurityGroups': ([str], True),
        'SubnetId': (str, True),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 8.1.0


from . import AWSObject
from . import AWSProperty


class Target(AWSProperty):
    props = {
        'TargetAddress': (str, False),
        'TargetType': (str, False),
    }


class NotificationRule(AWSObject):
    resource_type = "AWS::CodeStarNotifications::NotificationRule"

    props = {
        'DetailType': (str, True),
        'EventTypeIds': ([str], True),
        'Name': (str, True),
        'Resource': (str, True),
        'Status': (str, False),
        'Tags': (dict, False),
        'Targets': ([Target], True),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 3.3.0


from . import AWSObject
from . import AWSProperty
from troposphere import Tags
from .validators import boolean


class DeliveryOptions(AWSProperty):
    props = {
        'SendingPoolName': (str, False),
    }


class ReputationOptions(AWSProperty):
    props = {
        'ReputationMetricsEnabled': (boolean, False),
    }


class SendingOptions(AWSProperty):
    props = {
        'SendingEnabled': (boolean, False),
    }


class TrackingOptions(AWSProperty):
    props = {
        'CustomRedirectDomain': (str, False),
    }


class ConfigurationSet(AWSObject):
    resource_type = "AWS::PinpointEmail::ConfigurationSet"

    props = {
        'DeliveryOptions': (DeliveryOptions, False),
        'Name': (str, True),
        'ReputationOptions': (ReputationOptions, False),
        'SendingOptions': (SendingOptions, False),
        'Tags': (Tags, False),
        'TrackingOptions': (TrackingOptions, False),
    }


class DimensionConfiguration(AWSProperty):
    props = {
        'DefaultDimensionValue': (str, True),
        'DimensionName': (str, True),
        'DimensionValueSource': (str, True),
    }


class CloudWatchDestination(AWSProperty):
    props = {
        'DimensionConfigurations': ([DimensionConfiguration], False),
    }


class KinesisFirehoseDestination(AWSProperty):
    props = {
        'DeliveryStreamArn': (str, True),
        'IamRoleArn': (str, True),
    }


class PinpointDestination(AWSProperty):
    props = {
        'ApplicationArn': (str, False),
    }


class SnsDestination(AWSProperty):
    props = {
        'TopicArn': (str, True),
    }


class EventDestination(AWSProperty):
    props = {
        'CloudWatchDestination': (CloudWatchDestination, False),
        'Enabled': (boolean, False),
        'KinesisFirehoseDestination': (KinesisFirehoseDestination, False),
        'MatchingEventTypes': ([str], True),
        'PinpointDestination': (PinpointDestination, False),
        'SnsDestination': (SnsDestination, False),
    }


class ConfigurationSetEventDestination(AWSObject):
    resource_type = "AWS::PinpointEmail::ConfigurationSetEventDestination"

    props = {
        'ConfigurationSetName': (str, True),
        'EventDestination': (EventDestination, False),
        'EventDestinationName': (str, True),
    }


class DedicatedIpPool(AWSObject):
    resource_type = "AWS::PinpointEmail::DedicatedIpPool"

    props = {
        'PoolName': (str, False),
        'Tags': (Tags, False),
    }


class MailFromAttributes(AWSProperty):
    props = {
        'BehaviorOnMxFailure': (str, False),
        'MailFromDomain': (str, False),
    }


class Identity(AWSObject):
    resource_type = "AWS::PinpointEmail::Identity"

    props = {
        'DkimSigningEnabled': (boolean, False),
        'FeedbackForwardingEnabled': (boolean, False),
        'MailFromAttributes': (MailFromAttributes, False),
        'Name': (str, True),
        'Tags': (Tags, False),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


from . import AWSObject
from . import AWSProperty
from troposphere import Tags
from .validators import boolean
from .validators import double


def resolver_kind_validator(x):
    valid_types = ["UNIT", "PIPELINE"]
    if x not in valid_types:
        raise ValueError("Kind must be one of: %s" % ", ".join(valid_types))
    return x


class ApiCache(AWSObject):
    resource_type = "AWS::AppSync::ApiCache"

    props = {
        'ApiCachingBehavior': (str, True),
        'ApiId': (str, True),
        'AtRestEncryptionEnabled': (boolean, False),
        'TransitEncryptionEnabled': (boolean, False),
        'Ttl': (double, True),
        'Type': (str, True),
    }


class ApiKey(AWSObject):
    resource_type = "AWS::AppSync::ApiKey"

    props = {
        'ApiId': (str, True),
        'Description': (str, False),
        'Expires': (double, False),
    }


class DeltaSyncConfig(AWSProperty):
    props = {
        'BaseTableTTL': (str, True),
        'DeltaSyncTableName': (str, True),
        'DeltaSyncTableTTL': (str, True),
    }


class DynamoDBConfig(AWSProperty):
    props = {
        'AwsRegion': (str, True),
        'DeltaSyncConfig': (DeltaSyncConfig, False),
        'TableName': (str, True),
        'UseCallerCredentials': (boolean, False),
        'Versioned': (boolean, False),
    }


class ElasticsearchConfig(AWSProperty):
    props = {
        'AwsRegion': (str, True),
        'Endpoint': (str, True),
    }


class AwsIamConfig(AWSProperty):
    props = {
        'SigningRegion': (str, False),
        'SigningServiceName': (str, False),
    }


class AuthorizationConfig(AWSProperty):
    props = {
        'AuthorizationType': (str, True),
        'AwsIamConfig': (AwsIamConfig, False),
    }


class HttpConfig(AWSProperty):
    props = {
        'AuthorizationConfig': (AuthorizationConfig, False),
        'Endpoint': (str, True),
    }


class LambdaConfig(AWSProperty):
    props = {
        'LambdaFunctionArn': (str, True),
    }


class RdsHttpEndpointConfig(AWSProperty):
    props = {
        'AwsRegion': (str, True),
        'AwsSecretStoreArn': (str, True),
        'DatabaseName': (str, False),
        'DbClusterIdentifier': (str, True),
        'Schema': (str, False),
    }


class RelationalDatabaseConfig(AWSProperty):
    props = {
        'RdsHttpEndpointConfig': (RdsHttpEndpointConfig, False),
        'RelationalDatasourceType': (str, True),
    }


class DataSource(AWSObject):
    resource_type = "AWS::AppSync::DataSource"

    props = {
        'ApiId': (str, True),
        'Description': (str, False),
        'DynamoDBConfig': (DynamoDBConfig, False),
        'ElasticsearchConfig': (ElasticsearchConfig, False),
        'HttpConfig': (HttpConfig, False),
        'LambdaConfig': (LambdaConfig, False),
        'Name': (str, True),
        'RelationalDatabaseConfig': (RelationalDatabaseConfig, False),
        'ServiceRoleArn': (str, False),
        'Type': (str, True),
    }


class FunctionConfiguration(AWSObject):
    resource_type = "AWS::AppSync::FunctionConfiguration"

    props = {
        'ApiId': (str, True),
        'DataSourceName': (str, True),
        'Description': (str, False),
        'FunctionVersion': (str, True),
        'Name': (str, True),
        'RequestMappingTemplate': (str, False),
        'RequestMappingTemplateS3Location': (str, False),
        'ResponseMappingTemplate': (str, False),
        'ResponseMappingTemplateS3Location': (str, False),
    }


class CognitoUserPoolConfig(AWSProperty):
    props = {
        'AppIdClientRegex': (str, False),
        'AwsRegion': (str, False),
        'UserPoolId': (str, False),
    }


class OpenIDConnectConfig(AWSProperty):
    props = {
        'AuthTTL': (double, False),
        'ClientId': (str, False),
        'IatTTL': (double, False),
        'Issuer': (str, False),
    }


class AdditionalAuthenticationProvider(AWSProperty):
    props = {
        'AuthenticationType': (str, True),
        'OpenIDConnectConfig': (OpenIDConnectConfig, False),
        'UserPoolConfig': (CognitoUserPoolConfig, False),
    }


class LogConfig(AWSProperty):
    props = {
        'CloudWatchLogsRoleArn': (str, False),
        'ExcludeVerboseContent': (boolean, False),
        'FieldLogLevel': (str, False),
    }


class UserPoolConfig(AWSProperty):
    props = {
        'AppIdClientRegex': (str, False),
        'AwsRegion': (str, False),
        'DefaultAction': (str, False),
        'UserPoolId': (str, False),
    }


class GraphQLApi(AWSObject):
    resource_type = "AWS::AppSync::GraphQLApi"

    props = {
        'AdditionalAuthenticationProviders':
            ([AdditionalAuthenticationProvider], False),
        'AuthenticationType': (str, True),
        'LogConfig': (LogConfig, False),
        'Name': (str, True),
        'OpenIDConnectConfig': (OpenIDConnectConfig, False),
        'Tags': (Tags, False),
        'UserPoolConfig': (UserPoolConfig, False),
    }


class GraphQLSchema(AWSObject):
    resource_type = "AWS::AppSync::GraphQLSchema"

    props = {
        'ApiId': (str, True),
        'Definition': (str, False),
        'DefinitionS3Location': (str, False),
    }


class CachingConfig(AWSProperty):
    props = {
        'CachingKeys': ([str], False),
        'Ttl': (double, False),
    }


class PipelineConfig(AWSProperty):
    props = {
        'Functions': ([str], False),
    }


class LambdaConflictHandlerConfig(AWSProperty):
    props = {
        'LambdaConflictHandlerArn': (str, False),
    }


class SyncConfig(AWSProperty):
    props = {
        'ConflictDetection': (str, True),
        'ConflictHandler': (str, False),
        'LambdaConflictHandlerConfig': (LambdaConflictHandlerConfig, False),
    }


class Resolver(AWSObject):
    resource_type = "AWS::AppSync::Resolver"

    props = {
        'ApiId': (str, True),
        'CachingConfig': (CachingConfig, False),
        'DataSourceName': (str, False),
        'FieldName': (str, True),
        'Kind': (resolver_kind_validator, False),
        'PipelineConfig': (PipelineConfig, False),
        'RequestMappingTemplate': (str, False),
        'RequestMappingTemplateS3Location': (str, False),
        'ResponseMappingTemplate': (str, False),
        'ResponseMappingTemplateS3Location': (str, False),
        'SyncConfig': (SyncConfig, False),
        'TypeName': (str, True),
    }



from . import AWSProperty, AWSAttribute, validate_pausetime
from .validators import positive_integer, integer, boolean


class AutoScalingRollingUpdate(AWSProperty):
    props = {
        'MaxBatchSize': (positive_integer, False),
        'MinInstancesInService': (integer, False),
        'MinSuccessfulInstancesPercent': (integer, False),
        'PauseTime': (validate_pausetime, False),
        'SuspendProcesses': ([str], False),
        'WaitOnResourceSignals': (boolean, False),
    }


class AutoScalingScheduledAction(AWSProperty):
    props = {
        'IgnoreUnmodifiedGroupSizeProperties': (boolean, False),
    }


class AutoScalingReplacingUpdate(AWSProperty):
    props = {
        'WillReplace': (boolean, False),
    }


class CodeDeployLambdaAliasUpdate(AWSProperty):
    props = {
        'AfterAllowTrafficHook': (str, False),
        'ApplicationName': (boolean, True),
        'BeforeAllowTrafficHook': (str, False),
        'DeploymentGroupName': (boolean, True),
    }


class UpdatePolicy(AWSAttribute):
    props = {
        'AutoScalingRollingUpdate': (AutoScalingRollingUpdate, False),
        'AutoScalingScheduledAction': (AutoScalingScheduledAction, False),
        'AutoScalingReplacingUpdate': (AutoScalingReplacingUpdate, False),
        'CodeDeployLambdaAliasUpdate': (CodeDeployLambdaAliasUpdate, False),
        'UseOnlineResharding': (boolean, False),
    }


class ResourceSignal(AWSProperty):
    props = {
        'Count': (positive_integer, False),
        'Timeout': (validate_pausetime, False),
    }


class AutoScalingCreationPolicy(AWSProperty):
    props = {
        'MinSuccessfulInstancesPercent': (integer, False),
    }


class CreationPolicy(AWSAttribute):
    props = {
        'AutoScalingCreationPolicy': (AutoScalingCreationPolicy, False),
        'ResourceSignal': (ResourceSignal, True),
    }




from __future__ import print_function
import argparse
import sys
import time
from ast import literal_eval

try:
    import boto
    import boto.cloudformation
except ImportError:
    print("boto 2.8.0+ is required")
    sys.exit(1)

from troposphere.utils import tail


def upload_template_to_s3(conn, region, bucket_name, key_name, template):
    cloudformation_bucket = conn.lookup(bucket_name)
    if not cloudformation_bucket:
        if region == 'us-east-1':
            region = boto.s3.connection.Location.DEFAULT
        elif region == 'eu-west-1':
            region = boto.s3.connection.Location.EU

        cloudformation_bucket = conn.create_bucket(bucket_name,
                                                   location=region)
    key = cloudformation_bucket.new_key(key_name)
    key.set_contents_from_string(template)
    cfn_template_url = "https://s3.amazonaws.com/%s/%s" % (
        bucket_name, key_name)
    return cfn_template_url


def validate_template(conn, template=None, url=None):
    """Validate the template"""
    print("Validating Template")
    try:
        if url:
            conn.validate_template(template_url=url)
        elif template:
            conn.validate_template(template_body=template)
        else:
            print("Error: Must provide a template or template url")
            print("Exiting...")
            sys.exit(1)
    except boto.exception.BotoServerError as e:
        # XXX - need to figure out why this isn't getting parsed from boto.
        print("Error: %s" %
              (literal_eval(e.error_message)['Error']['Message'],))
        print("Exiting...")
        sys.exit(1)
    print("Template Validated")


def create_stack(
    conn,
    stackname,
    template=None,
    url=None,
    params=None,
    capabilities=None,
    update=False,
):
    kwargs = dict(
        template_body=template,
        template_url=url,
        parameters=params,
        capabilities=capabilities,
    )
    validate_template(conn, template=template, url=url)

    try:
        if update:
            stack_id = conn.update_stack(stackname, **kwargs)
            action = 'Updated'
        else:
            stack_id = conn.create_stack(stackname, **kwargs)
            action = 'Created'
        print("%s stack %s: %s" % (action, stackname, stack_id))
    except boto.exception.BotoServerError as e:
        # XXX - need to figure out why this isn't getting parsed from boto.
        print("Error: %s" %
              (literal_eval(e.error_message)['Error']['Message'],))
        print("Exiting...")
        sys.exit(1)


def build_s3_name(stack_name):
    timestamp = time.strftime('%Y-%m-%dT%H:%M:%SZ', time.gmtime())
    name = stack_name
    if stack_name.endswith('.json'):
        name = stack_name[:-5]
    return '%s-%s.json' % (name, timestamp)


def describe_resources(conn, stackname):
    if stackname is not None:
        print(conn.describe_stack_resources(stackname))
    else:
        stacks = conn.describe_stacks()
        for stack in stacks:
            print("Resources for stack {0}:".format(stack.stack_name))
            print(conn.describe_stack_resources(stack.stack_name))


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("-c", "--create", help="Create stack using template")
    parser.add_argument("-u", "--update", action='store_true',
                        help="Update stack instead of creating")
    parser.add_argument("-b", "--bucket", dest="s3bucket",
                        help="Upload template to S3 bucket")
    parser.add_argument("-d", "--debug", action='store_true',
                        help="Turn on boto debug logging")
    parser.add_argument("-n", "--name", dest="s3name",
                        help="Template name in S3 bucket")
    parser.add_argument("-p", "--parameter", dest="params", action='append',
                        help="stack parameters in key=value form")
    parser.add_argument("-r", "--region", default="us-east-1",
                        help="Region (default %(default)s)")
    parser.add_argument("-s", "--s3-region", default=None,
                        dest="s3region",
                        help="S3 Region (default us-east-1)")
    parser.add_argument("-R", "--resources", action='store_true',
                        help="describe stack resources, list resources for "
                             "all stacks if no stack is specified")
    parser.add_argument("-t", "--tail", action='store_true',
                        help="tail event log")
    parser.add_argument(
        "-C",
        "--capability",
        dest="capabilities",
        action='append',
        help="Capability to allow in the stack",
    )
    parser.add_argument("stack", nargs='?')
    values = parser.parse_args()

    if values.params:
        values.params = [x.split('=') for x in values.params]
    else:
        values.params = []

    if values.debug:
        import logging
        logging.basicConfig(filename="boto.log", level=logging.DEBUG)

    if not values.s3region:
        values.s3region = values.region

    conn = boto.cloudformation.connect_to_region(values.region)

    if values.create:
        # Read in the template file
        template = open(values.create).read()

        # If needed, build an S3 name (key)
        if values.s3bucket and not values.s3name:
            values.s3name = build_s3_name(values.create)

        kwargs = dict(
            template=template,
            params=values.params,
            capabilities=values.capabilities,
            update=values.update,
        )

        if values.s3bucket:
            # Upload to S3 and create the stack
            s3conn = boto.s3.connect_to_region(values.s3region)
            url = upload_template_to_s3(
                s3conn, values.region, values.s3bucket,
                values.s3name, template)
            kwargs['url'] = url
            del kwargs['template']

        create_stack(conn, values.stack, **kwargs)

    if values.resources:
        describe_resources(conn, values.stack)

    if values.tail:
        tail(conn, values.stack)
